/**
 Author: Daniel Archambault
 Email : archam@cs.ubc.ca
 Last modification : 08/12/2004
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by  
 the Free Software Foundation; either version 2 of the License, or     
 (at your option) any later version.
*/
/**
 * implementation of Resolve Overlap.  For nodes, history, 
 * author and licence please see the header file ResolveOverlap.h
 */

#include <math.h>
#include <vector>
#include "AceTest.h"

LAYOUTPLUGIN(AceTest,"Ace Test","Daniel Archambault","08/12/2004","Ok","0","1");

using namespace std;

AceTest::AceTest(const PropertyContext &context):Layout(context){}
AceTest::~AceTest() {}

#define CONST_THRESH (0.35)
inline bool isConstant (unsigned int deg, unsigned int numNodes) {
  return deg < CONST_THRESH*numNodes;
}//end isConstant

#define VARIENCE_THRESH (0.6)

bool AceTest::run() {
  //back up the node positions
  Iterator <node> *nodes = superGraph->getNodes();
  LayoutProxy *pos = superGraph->getProperty <LayoutProxy> ("viewLayout");
  while (nodes->hasNext()) {
    node curNode = nodes->next();
    layoutProxy->setNodeValue (curNode, pos->getNodeValue (curNode));
  } delete nodes; //end while

  //Compute the the average degree
  nodes = superGraph->getNodes();
  unsigned int sum = 0;
  unsigned int maxDeg = 0;
  while (nodes->hasNext()) {
    node curNode = nodes->next();
    unsigned int deg = superGraph->deg (curNode);
    if (deg > maxDeg) maxDeg = deg; 
    if (isConstant (deg, superGraph->numberOfNodes()))
      sum += deg;
    else {
      cerr << "Node " << curNode.id << " is of non-constant degree "
	   << deg << " out of " << superGraph->numberOfNodes() << " nodes" 
	   << endl;
      return false;
    }//end else
  } delete nodes; //end while

  //compute the average
  double average = sum / static_cast <double> (superGraph->numberOfNodes());
  cout << average << endl;

  //cout the max degree
  cerr << "The max degree of a node in this graph is " << maxDeg << endl;

  //Compute the varience
  nodes = superGraph->getNodes();
  double sqrVarSum = 0;
  while (nodes->hasNext()) {
    node curNode = nodes->next();
    double diff = superGraph->deg (curNode) - average;
    sqrVarSum += diff*diff;
  } delete nodes; //end while
  double sqrVar = 
    sqrVarSum / static_cast <double> (superGraph->numberOfNodes() - 1);
  if (sqrVar > VARIENCE_THRESH) {
    cerr << "The squared variance in degree is too large. " 
	 << sqrVar << " > " << (VARIENCE_THRESH) << endl;
    return false;
  }//end if

  //layout using ACE
  LayoutProxy localLayout (superGraph);
  string errMsg;
  superGraph->computeProperty ("Embedder (ACE)", &localLayout, errMsg,
			       pluginProgress, dataSet);
  nodes = superGraph->getNodes();
  while (nodes->hasNext()) {
    node curNode = nodes->next();
    layoutProxy->setNodeValue (curNode, localLayout.getNodeValue (curNode));
  } delete nodes; //end while
  return true;
}//end run
