//-*-c++-*-
#ifndef _BICONFILTER_H
#define _BICONFILTER_H

#include <tulip/TulipPlugin.h>
/** \addtogroup metric */
/*@{*/
/// StrongComponent.h - An implementation of the Stongly connected component algorithm.
/** This plugin is an implementation of the Stongly connected algorithm.
 *
 *  HISTORY:
 *
 *  01/12/1999 Verson 0.0.1: Initial release
 *
 *  NOTES:
 *
 *  This algorithm assigns to each node a value defined as following : If two nodes are in the same
 *  connected component they have the same value else they have a 
 *  different value.
 *
 *  AUTHOR:
 *
 *  David Auber University Bordeaux I France: Email:auber@tulip-software.com
 *
 *  LICENCE:
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.
 *
 */
#include <vector>
#include <ext/hash_map>
#include <tulip/MutableContainer.h>
using namespace std;
class BiconFilter:public Metric
{ 
public:
  BiconFilter(const PropertyContext &);
  ~BiconFilter();
  bool run();
private:
  inline void copyNodeProperties (node oldNode, node newNode);
  inline void copyEdgeProperties (edge oldEdge, edge newEdge);
  void fixArticulationVertex (node n, vector <int> &non_bridge_labels,
			      __gnu_cxx::hash_map <int, unsigned int> 
			      &label_count,
			      MutableContainer <bool> &leaveEdges,
			      MetricProxy &tempMetric);
};
/*@}*/
#endif
