/**
 Author: David Auber / Daniel Archambault
 Email : auber@labri.fr, archam@cs.ubc.ca
 Last modification : 25/11/2004
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by  
 the Free Software Foundation; either version 2 of the License, or     
 (at your option) any later version.
*/

/**
 * implementation of Circular (which is bubble circular).  For nodes, history, 
 * author and licence please see the header file BubbleCircular.h
 */

#include <math.h>
#include "BubbleCircular.h"

LAYOUTPLUGIN(BubbleCircular,"Circular","David Auber/ Daniel Archambault","25/11/2004","Ok","0","1");

using namespace std;

BubbleCircular::BubbleCircular(const PropertyContext &context):Layout(context){}

BubbleCircular::~BubbleCircular() {}

//this inline function computes the radius size given a size
inline double computeRad (const Size &s) {
  return sqrt (s.getW()*s.getW()/4.0 + s.getH()*s.getH()/4.0);
}//end computeRad

bool BubbleCircular::run() {
  Size resultSize;
  double sizeTm;

  //sizes of the nodes
  SizesProxy *nodeSizes = superGraph->getProperty<SizesProxy>("viewSize");

  //compute the sum of radii and the max radius of the graph
  double sumOfRad = 0;
  double maxRad = 0;
  node maxRadNode;
  Iterator<node> *itN=superGraph->getNodes();
  while(itN->hasNext()) {
    node itn=itN->next();
    double rad = computeRad (nodeSizes->getNodeValue(itn));
    sumOfRad += rad;
    if (maxRad < rad) {
      maxRad = rad;
      maxRadNode = itn;
    }//end if
  } delete itN;

  //with two nodes, lay them on a line max rad appart
  if (superGraph->numberOfNodes() <= 2) {
    //set the (max 2) nodes maxRad appart
    itN = superGraph->getNodes();
    double xcoord = maxRad/2.0;
    while (itN->hasNext ()) {
      node itn = itN->next();
      layoutProxy->setNodeValue (itn, Coord (xcoord, 0, 0));
      xcoord *= -1;
    } delete itN;
  }//end if

  //with more nodes, things get a little more complicated...
  else {
    //this is the current angle
    double gamma = 0;

    //if the ratio of maxRad/sumOfRad > .5, we need to ajust angles
    bool angleAdjust = false;
    if (maxRad/sumOfRad > 0.5) {
      sumOfRad -= maxRad;
      angleAdjust = true;
    }//end if

    //go through the nodes
    itN = superGraph->getNodes();
    while (itN->hasNext ()) {
      //get the node and its size
      node itn = itN->next();
      Size curNodeSize = nodeSizes->getNodeValue (itn);
      
      //compute the radius to ensure non overlap.  If adjustment to 
      //ensure no angle greater than pi done, detect it.
      double nodeRad = computeRad (curNodeSize);
      double halfAngle = 
	(nodeRad/sumOfRad)*((angleAdjust) ? M_PI/2.0 : M_PI);
      double rayon = nodeRad/sin(halfAngle);

      //if this node was the node that took up more than half the circle,
      //we complet the adjustment to make sure that it does not.
      if (angleAdjust && (maxRadNode == itn)) {
	halfAngle = M_PI/2.0;
	rayon = nodeRad;
      }//end if

      //compute the position of the node.
      gamma += halfAngle;
      layoutProxy->setNodeValue(itn, Coord(rayon*cos(gamma),
					   rayon*sin(gamma), 0));
      gamma += halfAngle;
    }//end while
    delete itN;
  }//end else
  return true;
} //end run





