/**
 Author: David Auber
 Email : auber@labri.fr
 Last modification : 20/08/2001
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by  
 the Free Software Foundation; either version 2 of the License, or     
 (at your option) any later version.
*/

/**
 * implementation of Regular Circular.  For nodes, history, 
 * author and licence please see the header file Circular.h
 */

#include <math.h>
#include "Circular.h"

LAYOUTPLUGIN(Circular,"Regular Circular","David Auber/ Daniel Archambault","25/11/2004","Ok","0","1");

using namespace std;

Circular::Circular(const PropertyContext &context):Layout(context){}

Circular::~Circular() {}

//this inline function computes the radius size given a size
inline double computeRad (const Size &s) {
  return sqrt (s.getW()*s.getW()/4.0 + s.getH()*s.getH()/4.0);
}//end computeRad

bool Circular::run() {
  Size resultSize;
  double sizeTm;

  //sizes of the nodes
  SizesProxy *nodeSizes = superGraph->getProperty<SizesProxy>("viewSize");

  //compute the sum of radii and the max radius of the graph
  double sumOfRad = 0;
  double maxRad = 0;
  Iterator<node> *itN=superGraph->getNodes();
  while(itN->hasNext()) {
    node itn=itN->next();
    double rad = computeRad (nodeSizes->getNodeValue(itn));
    sumOfRad += rad;
    if (maxRad < rad) maxRad = rad;
  } delete itN;

  //with two nodes, lay them on a line
  if (superGraph->numberOfNodes() < 2) {
    //set the (max 2) nodes maxRad appart
    itN = superGraph->getNodes();
    double xcoord = maxRad/2.0;
    while (itN->hasNext ()) {
      node itn = itN->next();
      layoutProxy->setNodeValue (itn, Coord (xcoord, 0, 0));
      xcoord *= -1;
    } delete itN;
  }//end if

  //with more nodes, things get a little more complicated...
  else {
    //this is an estimation of the radius of the circle
    double rayon = sumOfRad/M_PI;

    //when the range of node sizes is large the sum of the
    //radii is not a good estimation of the ideal radius.
    //just take the maximum
    double rayonIdeal = (rayon < maxRad) ? maxRad : rayon;

    //this is the current angle
    double gamma = 0;

    //sort by the depth
    itN = superGraph->getNodes();
    while (itN->hasNext ()) {
      node itn = itN->next ();
      Size curNodeSize = nodeSizes->getNodeValue (itn);
      gamma += M_PI*(computeRad (curNodeSize)/sumOfRad);
      layoutProxy->setNodeValue(itn, Coord(rayonIdeal*cos(gamma),
					   rayonIdeal*sin(gamma), 0));
      gamma += M_PI*(computeRad (curNodeSize)/sumOfRad);
    }//end while
    delete itN;
  }//end else
  return true;
}





