//-*-c++-*-
#ifndef _COMPACT_FINAL_LAYOUT_H
#define _COMPACT_FINAL_LAYOUT_H

#include <tulip/TulipPlugin.h>

/** \addtogroup layout */
/*@{*/
/// Circular.h - An implementation of a circurlar layout.
/** 
 * This plugin is an implementation of a circular layout
 * All nodes are palced arbitrary on circle. 
 *
 * HISTORY:
 *
 * 01/12/99 Verson 0.0.1: Initial release
 *
 * NOTES:
 *
 * This work on general graphs.
 * Let n be the number of nodes, the algorithm complexity is in O(n).
 *
 * AUTHOR:
 *
 * David Auber University Bordeaux I France: Email:auber@tulip-software.com
 */

#include <vector>
#include <BoundingObjects.h>

using namespace std;

class CompactFinalLayout:public Layout { 
public:
  CompactFinalLayout(const PropertyContext &);
  ~CompactFinalLayout();
  bool run();
  bool check(std::string &);
  void reset();

private:
  //a list of bounding objects for nodes in the graph
  BoundaryTable *boundaryTable;

  //a list of the graphs and if shrink resolve has succeeded on them.
  shrinkResTable *shrinkResolveTable;

  //a list of graph types where we need to consider every pair of nodes.
  vector <string> *compactAllPairs;

  //A function for compacting a particular level.  It returns true if
  //it succeeds
  bool compactLevel (SuperGraph *graph, string &compactType);

  //Recursively calls CompactFinalLayout to compact all the subgraphs and
  //the final graph.
  bool recurseOnSubGraphs (SuperGraph *graph, string &compactType);

};
/*@}*/
#endif
