//-*-c++-*-
#ifndef _COMPACT_TREE_H
#define _COMPACT_TREE_H

#include <tulip/TulipPlugin.h>

/** \addtogroup layout */
/*@{*/
/// Circular.h - An implementation of a circurlar layout.
/** 
 * This plugin is an implementation of a circular layout
 * All nodes are palced arbitrary on circle. 
 *
 * HISTORY:
 *
 * 01/12/99 Verson 0.0.1: Initial release
 *
 * NOTES:
 *
 * This work on general graphs.
 * Let n be the number of nodes, the algorithm complexity is in O(n).
 *
 * AUTHOR:
 *
 * David Auber University Bordeaux I France: Email:auber@tulip-software.com
 */

#include <vector>
using namespace std;

class CompactTree:public Layout { 
public:
  CompactTree(const PropertyContext &);
  ~CompactTree();
  bool run();
  bool check(std::string &errMsg);
  void reset();
private:
  //The following function computes the maximum shrinking factor that
  //can be applied to one node of a triple of nodes so that the triple
  //does not overlap.  The 'root' Coord is the coordinates of the root.  The
  //'leaf' Coord is the coordinates of the leaf.  The 'd' Coord is the
  //position of the node being moved in.  The doubles are the radius of
  //the root, leaf and d nodes respectively.
  inline double compactD (Coord root, Coord leaf, Coord d,
			  double root_rad, double leaf_rad, 
			  double d_rad);
  
  //This small function translates all the dependents of the passed
  //node 'n' by 'trans'
  void transDependents (node &n, Coord trans);

  //computes the new centre and radius for a branch and set them.  
  //The branch is rooted at "root" and has nodes "compactedNodes"
  void setCentreAndRad (node root, 
			vector <node> &alreadyCompacted);

  //This function compacts a specific branch areound a node, updating
  //its radius and centre accordingly.
  void compactBranch (node &root);

  //The following function fixes the tree so that the new root is
  //the actual root
  void fixTree (node cur_root, node came_from);

  //The following function makes the node of highest degree the root
  //of the tree
  node makeHighestDegreeRoot ();
};
/*@}*/
#endif
