#include <tulip/TlpTools.h>
#include <tulip/StableIterator.h>
#include <tulip/TulipPlugin.h>
#include <tulip/ForEach.h>
#include <iostream>

using namespace std;

///
class Singletons : public Metric { 

public:

  Singletons(const PropertyContext &context):Metric(context){};

  //=======================================================================
  /**
   * This function is simply the entry point for the plugin.  It finds
   * the singletons
   */
  bool run() {
    SuperGraph *graph=tlp::newCloneSubGraph(superGraph);
    string errMsg;    
    DataSet localData;
    MetricProxy trees (graph);
    localData.set ("Non-Tree Roots", true);
    
    //find all tree components and put in subgraphs
    bool success = 
      graph->computeProperty("Connected_Tree Component 2", 
			     &trees, errMsg, NULL, &localData);
    if (!success) {
      cerr << "Error in metric plugin Connected_Tree Component 2 " << endl 
	   << errMsg << endl;
      return false;
    }//end if
    localData.set ("Metric", &trees);
    success = 
      tlp::clusterizeGraph (graph, errMsg, &localData, "Equal Value");
    if (!success) {
      cerr << "Error in Equal Value Clustering " << endl << errMsg << endl;
      return false;
    }//end if

    //keep only those who are singleton collections:  trees of exactly depth 
    //one.
    metricProxy->setAllNodeValue (0);
    unsigned int singletonIdent = 1;
    Iterator<SuperGraph *> *subgraphs = graph->getSubGraphs();
    while (subgraphs->hasNext()) {
      SuperGraph *curSubGraph = subgraphs->next();
      node root;
      tlp::getSource (curSubGraph, root);
      bool hasSingleton = false;
      node curNode;
      forEach (curNode, curSubGraph->getInOutNodes (root))
	hasSingleton |= (curSubGraph->deg(curNode) == 1);

      //if we have a singleton identify it
      if (hasSingleton) {
	forEach (curNode, curSubGraph->getInOutNodes (root)) {
	  if (curSubGraph->deg(curNode) == 1)
	    metricProxy->setNodeValue (curNode, singletonIdent);
	}//end forEach
	metricProxy->setNodeValue (root, singletonIdent);
	++singletonIdent;
      }//end if
    } delete subgraphs;
    superGraph->delAllSubGraphs(graph);
    return true;
  }//end run
  //=======================================================================
};

METRICPLUGIN(Singletons,"Singletons", "Daniel Archambault","03/01/2005","Alpha","0","1");
