//-*-c++-*-
#ifndef _RESOLVE_OVERLAP_H
#define _RESOLVE_OVERLAP_H

#include <tulip/TulipPlugin.h>
#include <tulip/MutableContainer.h>

/** \addtogroup layout */
/*@{*/
/// ReslolveOverlap.h - Resolves all node overlaps.
/** 
 * The algorithm finds and applies the minimum scale factor to ensure
 * that no two pairs of nodes overlap in the graph
 *
 * HISTORY:
 *
 * 25/11/2004 Verson 0.0.1: Initial release
 *
 * NOTES:
 *
 *   The algorithm works on general graphs.
 *   Let n be the number of nodes, the algorithm complexity is in O(n).
 *
 *   If two nodes start out at the same postion the algorithm cannot
 *   succeed.  It will return failure.
 *   
 * AUTHOR:
 *
 * Daniel Archambault, The University of British Columbia 
 * Email:archam@cs.ubc.ca
 *
 *  LICENCE:
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.
 *
 */

class ResolveOverlap:public Layout { 
public:
  ResolveOverlap(const PropertyContext &);
  ~ResolveOverlap();
  bool run();
private:
  //a private function centre the superGraph around the passed centre
  //node
  void centreGraph (node centre);
};
/*@}*/
#endif
