//-*-c++-*-
#ifndef _COMPACT_FINAL_LAYOUT_H
#define _COMPACT_FINAL_LAYOUT_H

#include <tulip/TulipPlugin.h>

/** \addtogroup layout */
/*@{*/
/// Circular.h - An implementation of a circurlar layout.
/** 
 * This plugin is an implementation of a circular layout
 * All nodes are palced arbitrary on circle. 
 *
 * HISTORY:
 *
 * 01/12/99 Verson 0.0.1: Initial release
 *
 * NOTES:
 *
 * This work on general graphs.
 * Let n be the number of nodes, the algorithm complexity is in O(n).
 *
 * AUTHOR:
 *
 * David Auber University Bordeaux I France: Email:auber@tulip-software.com
 */

#include <vector>
#include <BoundingObjects.h>

class OverlapsFinalLayout:public Sizes { 
public:
  OverlapsFinalLayout(const PropertyContext &);
  ~OverlapsFinalLayout();
  bool run();
  bool check(std::string &);
  void reset();

private:
  //a list of bounding objects for nodes in the graph
  BoundaryTable *boundaryTable;

  //a list of graphs and if shrink resolve succeeded on them
  shrinkResTable *shrinkResolveTable;

  //a list of graph types where we need to consider every pair of nodes.
  std::vector<std::string> *overlapsAllPairs;

  //A function for compacting a particular level.
  bool overlapsLevel (SuperGraph *graph, std::string &compactType);

  //A function to recurse of the subgraphs of the finalLayout
  bool recurseOnSubGraphs (SuperGraph *graph, std::string &compactType);
};
/*@}*/
#endif
