
#include <iostream>
#include <string>
#include <math.h>
#include <vector>
#include <ext/hash_map>

using namespace std;

#include "SingletonFilter.h"
#include <tulip/TlpTools.h>

METRICPLUGIN(SingletonFilter,"Singleton Filter","Daniel Archambault","01/11/2005","Alpha","0","1");

SingletonFilter::SingletonFilter(const PropertyContext &context):Metric(context) {}

SingletonFilter::~SingletonFilter(){}

bool SingletonFilter::run() {
  //use the Connected & Tree to detect the singletons.
  string errMsg;
  MetricProxy singletonMetric (superGraph); 
  bool success = superGraph->computeProperty ("Connected_Tree Component", 
					      &singletonMetric, errMsg);
  //if this did not run, the user better install 
  //TREE_COMPONENT_METRIC plugin
  if (!success) {
    cerr << "You cannot use this clustering algorithm without the "
	 << "\"Connected & Tree Component\" plugin" << endl;
    return false;
  }//end if

  //a list of booleans indicating if this node has changed its label
  __gnu_cxx::hash_map <unsigned int, bool> 
    changed_label (superGraph->numberOfNodes());
  Iterator <node> *nodes = superGraph->getNodes();
  while (nodes->hasNext()) {
    node curNode = nodes->next();
    changed_label [curNode.id] = false;
  }//end for i
  delete nodes;

  //for each of the nodes in the graph, determine if it is a singleton
  //and propagate labels accordingly
  nodes = superGraph->getNodes();
  while (nodes->hasNext()) {
    node curNode = nodes->next();

    //bin only singletons.  A singleton, is a degree one node in
    //the graph
    if (superGraph->deg (curNode) == 1) {

      //get only opposite node
      Iterator <node> *oppositeNode = superGraph->getInOutNodes(curNode);
      node opposite = oppositeNode->next();
      delete oppositeNode;

      //get only edge
      Iterator <edge> *edges = superGraph->getInOutEdges(curNode);
      edge onlyEdge = edges->next();
      delete edges;

      //flip the edge if pointing the wrong way
      if (superGraph->source (onlyEdge) == curNode)
	superGraph->reverse (onlyEdge);

      //check if the root has changed its label
      if (!changed_label [opposite.id]) {
	metricProxy->setNodeValue (opposite, 
				   singletonMetric.getNodeValue (curNode));
	changed_label [opposite.id] = true;
      }//end if
      metricProxy->setNodeValue (curNode, 
				 metricProxy->getNodeValue (opposite));
    }//end if
    
    //otherwise copy over the label from tree detection.
    else 
      metricProxy->setNodeValue (curNode,
				 singletonMetric.getNodeValue (curNode));
  } delete nodes; //end while

  //return success
  return true;
}








