
#include <iostream>
#include <string>

using namespace std;

#include "StrengthFilter.h"
#include <tulip/TlpTools.h>


METRICPLUGIN(StrengthFilter,"Strength Filter","Daniel Archambault","01/11/2005","Alpha","0","1");

StrengthFilter::StrengthFilter(const PropertyContext &context):Metric(context) {}

StrengthFilter::~StrengthFilter(){}

bool StrengthFilter::run() {
  //for error messages
  string errMsg;

  //clone the passed graph.
  SuperGraph *strengthGraph = tlp::newCloneSubGraph (superGraph);
  strengthGraph->setAttribute <string> ("name", "strength temp");

  //call the biconnected component metric on the clone and
  //get the resulting viewMetric
  MetricProxy *viewMetric = strengthGraph
    ->getLocalProperty <MetricProxy> ("viewMetric");
  bool success = 
    strengthGraph->computeProperty ("Strength Component", 
				    viewMetric, errMsg);
  if (!success) {
    cerr << "Error in Strength Filter" << endl;
    cerr << "Strength Componnent returned the following error:\n" 
	 << errMsg << endl;
    return false;
  }

  //perform equal value clustering
  DataSet localData;
  success =
    tlp::clusterizeGraph (strengthGraph, errMsg, &localData, "Equal Value");
  if (!success) {
    cerr << "Error in Strength Filter." << endl;
    cerr << "Equal Value clustering returned the following error:\n"
	 << errMsg << "\n(Is the plugin properly installed?)" << endl;
    return false;
  }//end if
  
  //get the maximum id on all nodes and the metricProxy
  int maxId = 0;
  SuperGraph *remainderBin = NULL;
  Iterator <SuperGraph *> *graphs = strengthGraph->getSubGraphs();
  while (graphs->hasNext()) {
    SuperGraph *curGraph = graphs->next();

    //go through the nodes in the sub graphs
    Iterator <node> *nodes = curGraph->getNodes();
    while (nodes->hasNext()) {
      node curNode = nodes->next();
      int metricValue = 
	static_cast <int> (floor (viewMetric->getNodeValue (curNode)));
      if (metricValue > maxId) maxId = metricValue;
      metricProxy->setNodeValue (curNode, metricValue);
    } delete nodes; //end while

    //if this graph is the remainder bin (all nodes no edges)
    //record a pointer to it
    if (curGraph->numberOfEdges() == 0) remainderBin = curGraph;
  } delete graphs; //end while

  //give all the nodes in the remainder bin different ids, greater
  //than maxId
  if (remainderBin != NULL) {
    Iterator <node> *nodes = remainderBin->getNodes();
    while (nodes->hasNext())
      metricProxy->setNodeValue (nodes->next(), ++maxId);
  }//end if

  //delete the temporary sub graph
  superGraph->delAllSubGraphs (strengthGraph);
  return true;
}//end run








