#include <math.h>
#include <stdio.h>
#include <sstream>
#include <string>
#include <list>
#include <map>
#include <iostream>
#include <tulip/SuperGraph.h>
#include <tulip/SelectionProxy.h>
#include <tulip/MetaGraphProxy.h>
#include <tulip/TlpTools.h>
#include <tulip/GraphMeasure.h>
#include <tulip/StableIterator.h>
#include "StrengthComponent.h"

METRICPLUGIN(StrengthComponent,"Strength Component","David Auber","27/01/2003","Alpha","0","1");
using namespace std;

//================================================================================
StrengthComponent::StrengthComponent(const PropertyContext &context):Metric(context) {}
//================================================================================
StrengthComponent::~StrengthComponent() {}
//================================================================================
double StrengthComponent::e(set<node> &U,set<node> &V) {
  set<node>::const_iterator itU;
  double result=0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    for (;itN->hasNext();) {
      node itn=itN->next();
      if (V.find(itn)!=V.end()) result+=1;
    }delete itN;
  }
  return result;
}
//==============================================================================
double StrengthComponent::e(set<node> &U) {
  set<node>::const_iterator itU;
  double result=0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    for (;itN->hasNext();) {
      node itn=itN->next();
      if (U.find(itn)!=U.end()) result+=1;
    }delete itN;
  }
  return result/2.0;
}
//==============================================================================
double StrengthComponent::s(set<node> &U,set<node> &V) {
  if ((U.size()==0) || (V.size()==0)) return 0;
  return (e(U,V) / double(U.size()*V.size()));
}
//==============================================================================
double StrengthComponent::s(set<node> &U) {
  if (U.size()<2) return 0;
  return  (e(U)) * 2.0 / double(U.size()*(U.size()-1.0));
}
//==============================================================================
double StrengthComponent::computeMQValue(vector<set<node> > partition) {
  double positive=0;
  for (unsigned int i=0;i<partition.size();++i) {
    positive+=s(partition[i]);
  }
  positive/=double(partition.size());
  double negative=0;
  for (unsigned int i=0;i<partition.size()-1;++i)
    for (unsigned int j=i+1;j<partition.size();++j) {
      negative+=s(partition[i],partition[j]);
    }
  if (partition.size()>1)
    negative/=double(partition.size()*(partition.size()-1)/2.0);
  return positive-negative;
}
//==============================================================================
void StrengthComponent::computeNodePartition(double threshold, vector< set<node> >&result) {
  //cerr << __PRETTY_FUNCTION__ << endl;
  string errMsg;
  SuperGraph *tmpGraph=tlp::newCloneSubGraph(superGraph);
  StableIterator<edge> itE(superGraph->getEdges());
  while (itE.hasNext()) {
    edge ite=itE.next();
    if (values->getEdgeValue(ite)<threshold) {
      if (superGraph->deg(superGraph->source(ite))>1 && superGraph->deg(superGraph->target(ite))>1)
	tmpGraph->delEdge(ite);
    }
  }
  // Select SubGraph singleton in superGraph
  /*
    set<node> singleton;
  StableIterator<node> itN(tmpGraph->getNodes());
  while (itN.hasNext()) {
    node itn=itN.next();
    if (tmpGraph->deg(itn)==0) singleton.insert(itn);
  }
  */
  // restore edges to reconnect singleton by computing induced subgraph 
  /*
  StableIterator<edge> itE2(superGraph->getEdges());
  while (itE2.hasNext()) {
    edge ite=itE2.next();
    if (singleton.find(superGraph->source(ite))!=singleton.end() && 
	singleton.find(superGraph->target(ite))!=singleton.end()) {
      tmpGraph->addEdge(ite);
    }
  }
  */
  //Extract connected componnent
  MetricProxy connected(tmpGraph); 
  tmpGraph->computeProperty("Connected Component", &connected, errMsg);
  //Put isolated nodes in the same cluster
  /*
    double val=-1;
    Iterator<node> *itN2 = tmpGraph->getNodes();
    while (itN2->hasNext()) {
    node itn=itN2->next();
    if (tmpGraph->deg(itn)==0) {
    if (val==-1) val=connected->getNodeValue(itn);
    else
    connected->setNodeValue(itn,val);
    }
    } delete itN2;
  */
  //Compute the node partition
  int index=0;
  map<double,int> resultIndex;
  Iterator<node> *itN2=tmpGraph->getNodes();
  while (itN2->hasNext()) {
    node itn=itN2->next();
    double val=connected.getNodeValue(itn);
    if (resultIndex.find(val)!=resultIndex.end())
      result[resultIndex[val]].insert(itn);
    else {
      set<node> tmp;
      result.push_back(tmp);
      resultIndex[val]=index;
      result[index].insert(itn);
      ++index;
    }
  } delete itN2;
  superGraph->delAllSubGraphs(tmpGraph);
}
//==============================================================================
double StrengthComponent::findBestThreshold(int numberOfSteps){
  //compute frequence histogram
  map<double, unsigned int> histo;
  Iterator<edge> *itE = superGraph->getEdges();
  double maxThreshold = values->getEdgeMin(superGraph);
  unsigned int maxThresholdFreq = 0;
  while(itE->hasNext()){
    edge e = itE->next();
    double val = values->getEdgeValue(e);
    if (val == 0) continue;
    if (histo.find(val)==histo.end())
      histo[val] = 0;
    histo[val] += 1;
    if (histo[val]>maxThresholdFreq){
      maxThresholdFreq = histo[val];
      maxThreshold = val;
    }
  }delete itE;
  return maxThreshold;
  /*
  double maxMQ=-2;
  double threshold = values->getEdgeMin(superGraph);
  double deltaThreshold = (values->getEdgeMax(superGraph)-values->getEdgeMin(superGraph))/double(numberOfSteps);
  for (double i = values->getEdgeMin(superGraph); i<values->getEdgeMax(superGraph); i+=deltaThreshold) {
    vector< set<node > > tmp;
    computeNodePartition(i, tmp);
    double mq = computeMQValue(tmp);
    if (mq>maxMQ) {
      threshold=i;
      maxMQ=mq;
    }
  }
  */
  /*
  cout << __PRETTY_FUNCTION__ << endl;
  cout << "\t" <<" Nb try : " << numberOfSteps << endl;
  cout << "\t" <<" Max MQ : " << maxMQ << endl << flush;
  cout << "\t" <<" Best threshold : " << threshold << endl << flush;
  */
  //  return threshold;
}
//==============================================================================
bool StrengthComponent::run() {
  bool result;
  string errMsg;
  values = new MetricProxy(superGraph);
  result = superGraph->computeProperty("Strength", values, errMsg);
  const unsigned int NB_TEST = 100;
  double threshold = findBestThreshold(NB_TEST);
  vector< set<node > > tmp;
  computeNodePartition(threshold, tmp);
  unsigned int count = 0;
  vector< set<node > >::const_iterator it;
  for (it=tmp.begin();it!=tmp.end();++it) {
    set<node>::const_iterator it2;
    for (it2=it->begin();it2!=it->end();++it2)
      metricProxy->setNodeValue(*it2,count);
    ++count;
  }
  return true;
}
//================================================================================
bool StrengthComponent::check(string &erreurMsg) {
  erreurMsg="";
  return true;
}
//================================================================================
void StrengthComponent::reset() {
}
//================================================================================
