#include <math.h>
#include <stdio.h>
#include <sstream>
#include <string>
#include <list>
#include <map>
#include <iostream>
#include <ext/hash_set>
#include <tulip/SuperGraph.h>
#include <tulip/SelectionProxy.h>
#include <tulip/MetaGraphProxy.h>
#include <tulip/TlpTools.h>
#include <tulip/GraphMeasure.h>
#include <tulip/StableIterator.h>
#include "StrengthComponent.h"

METRICPLUGIN(StrengthComponent,"Strength Component","David Auber","27/01/2003","Alpha","0","1");
using namespace std;

//================================================================================
StrengthComponent::StrengthComponent(const PropertyContext &context):Metric(context) {}
//================================================================================
StrengthComponent::~StrengthComponent() {}
//================================================================================
double StrengthComponent::computeMQValue(const vector<set<node> > & partition, SuperGraph *graph) {

  vector<unsigned int> nbIntraEdges(partition.size());
  for (unsigned int i = 0; i<partition.size(); ++i)
    nbIntraEdges[i] = 0;

  map<pair<unsigned int, unsigned int>, unsigned int > nbExtraEdges;

  MutableContainer<unsigned int> clusterId;  
  vector< set<node> >::const_iterator itPart = partition.begin();
  for (unsigned int i=0; itPart!=partition.end(); ++itPart, ++i) {
    set<node>::const_iterator itSet = itPart->begin();
    for (; itSet!=itPart->end(); ++itSet) {
      clusterId.set(itSet->id, i);
    }
  }
  
  Iterator<edge> *itE = graph->getEdges();
  while(itE->hasNext()) {
    edge e = itE->next();
    node n1; 
    node n2;
    if (graph->source(e).id < graph->target(e).id) {
      n1 = graph->source(e);
      n2 = graph->target(e);
    }
    else {
      n1 = graph->target(e);
      n2 = graph->source(e);
    }

    unsigned int n1ClustId = clusterId.get(n1.id);
    unsigned int n2ClustId = clusterId.get(n2.id);
    if ( n1ClustId == n2ClustId)
      nbIntraEdges[n1ClustId] += 1;
    else {
      pair<unsigned int, unsigned int> pp(n1ClustId, n2ClustId);
      if (nbExtraEdges.find(pp) != nbExtraEdges.end()) {
	nbExtraEdges[pp] += 1;
      }
      else {
	nbExtraEdges[pp] = 1;
      }
    }
  } delete itE;
  
  double positive = 0;
  for (unsigned int i=0; i<partition.size(); ++i) {
    if (partition[i].size() > 1)
      positive += 2.0 * double(nbIntraEdges[i]) / double(partition[i].size() * (partition[i].size() - 1));
  }
  positive /= double(partition.size());
  
  double negative=0;
  map<pair<unsigned int, unsigned int>, unsigned int >::const_iterator itMap = nbExtraEdges.begin();
  for (; itMap != nbExtraEdges.end(); ++itMap) {
    pair<unsigned int, unsigned int> pp = itMap->first;
    unsigned int val = itMap->second;
    if ( (partition[pp.first].size()!=0) && (partition[pp.second].size()!=0))
      negative += double(val) / double(partition[pp.first].size() * partition[pp.second].size());
  }
  if (partition.size()>1)
    negative /= double(partition.size()*(partition.size()-1)) / 2.0;
  double result = positive - negative;
  return result;
}
//==============================================================================
void StrengthComponent::computeNodePartition(double threshold, vector< set<node> >&result) {
  //cerr << __PRETTY_FUNCTION__ << endl;
  string errMsg;
  SuperGraph *tmpGraph=tlp::newCloneSubGraph(superGraph);
  StableIterator<edge> itE(superGraph->getEdges());
  while (itE.hasNext()) {
    edge ite=itE.next();
    if (values->getEdgeValue(ite)<threshold) {
      if (superGraph->deg(superGraph->source(ite))>1 && superGraph->deg(superGraph->target(ite))>1)
	tmpGraph->delEdge(ite);
    }
  }

  stdext::hash_set<node> singleton;
  Iterator<node> *itN = tmpGraph->getNodes();
  while (itN->hasNext()) {
    node itn = itN->next();
    if (tmpGraph->deg(itn)==0) singleton.insert(itn);
  }
  StableIterator<edge> itE2(superGraph->getEdges());
  while (itE2.hasNext()) {
    edge ite=itE2.next();
    if (singleton.find(superGraph->source(ite))!=singleton.end() && 
	singleton.find(superGraph->target(ite))!=singleton.end()) {
      tmpGraph->addEdge(ite);
    }
  }
  //Extract connected componnent
  MetricProxy connected(tmpGraph); 
  tmpGraph->computeProperty("Connected Component", &connected, errMsg);

  //Compute the node partition
  int index=0;
  map<double,int> resultIndex;
  Iterator<node> *itN2=tmpGraph->getNodes();
  while (itN2->hasNext()) {
    node itn=itN2->next();
    double val=connected.getNodeValue(itn);
    if (resultIndex.find(val)!=resultIndex.end())
      result[resultIndex[val]].insert(itn);
    else {
      set<node> tmp;
      result.push_back(tmp);
      resultIndex[val]=index;
      result[index].insert(itn);
      ++index;
    }
  } delete itN2;
  superGraph->delAllSubGraphs(tmpGraph);
}
//==============================================================================
double StrengthComponent::findBestThreshold(unsigned int numberOfSteps){
  //compute frequence histogram
  /*
    map<double, unsigned int> histo;
    Iterator<edge> *itE = superGraph->getEdges();
    double maxThreshold = values->getEdgeMin(superGraph);
    unsigned int maxThresholdFreq = 0;
    while(itE->hasNext()){
    edge e = itE->next();
    double val = values->getEdgeValue(e);
    if (val == 0) continue;
    if (histo.find(val)==histo.end())
    histo[val] = 0;
    histo[val] += 1;
    if (histo[val]>maxThresholdFreq){
    maxThresholdFreq = histo[val];
    maxThreshold = val;
    }
    }delete itE;
    return maxThreshold;
  */
  
  double maxMQ=-2;
  double threshold = values->getEdgeMin(superGraph);
  vector<unsigned int> nbClusters;
  vector<double> thresholds;
  vector<double> MQS;
  double deltaThreshold = (values->getEdgeMax(superGraph)-values->getEdgeMin(superGraph))/double(numberOfSteps);
  for (unsigned int i = 0; i < numberOfSteps; ++i) {
    double currentThreshold = 
      values->getEdgeMin(superGraph) + i*deltaThreshold;
    vector< set<node > > tmp;
    computeNodePartition(currentThreshold, tmp);
    nbClusters.push_back(tmp.size());
    thresholds.push_back(threshold);
    MQS.push_back(computeMQValue(tmp, superGraph));
    cerr << "size :" << tmp.size() << " MQ:" << MQS.back() << " threshold:" 
	 << currentThreshold << endl;
    //    double mq = computeMQValue(tmp);
    if (MQS.back()>maxMQ) { // && float(tmp.size())>sqrt(float(superGraph->numberOfNodes()))) {
      threshold = currentThreshold;
      maxMQ=MQS.back();
      cerr << "GOOD" << endl;
    }
  }
  /*
  cout << __PRETTY_FUNCTION__ << endl;
  cout << "\t" <<" Nb try : " << numberOfSteps << endl;
  cout << "\t" <<" Max MQ : " << maxMQ << endl << flush;
  cout << "\t" <<" Best threshold : " << threshold << endl << flush;
  */
  return threshold;
}
//==============================================================================
bool StrengthComponent::run() {
  bool result;
  string errMsg;
  values = new MetricProxy(superGraph);
  result = superGraph->computeProperty("Strength", values, errMsg);
  const unsigned int NB_TEST = 6;
  double threshold = findBestThreshold(NB_TEST);
  vector< set<node > > tmp;
  computeNodePartition(threshold, tmp);
  unsigned int count = 0;
  vector< set<node > >::const_iterator it;
  for (it=tmp.begin();it!=tmp.end();++it) {
    set<node>::const_iterator it2;
    for (it2=it->begin();it2!=it->end();++it2)
      metricProxy->setNodeValue(*it2,count);
    ++count;
  }
  return true;
}
//================================================================================
bool StrengthComponent::check(string &erreurMsg) {
  erreurMsg="";
  return true;
}
//================================================================================
void StrengthComponent::reset() {
}
//================================================================================
