#ifndef __HIERARCHY_MAP_H
#define __HIERARCHY_MAP_H

/// HierarchyMap.h
/** 
 *
 *  Defines required by Topological Feature Clustering and HierLayout
 *  to allow the two plugins to communicate
 *
 *  HISTORY:
 *
 *  The code was first implemented in Nov/Dec 2004 by Daniel Archambault.
 *
 *  NOTES:
 *
 *  AUTHOR:
 *
 *  Daniel Archambault, University of British Columbia, CA: 
 *  Email: archam@cs.ubc.ca
 *  Version 0.1: 08 December 2004: Initial version
 *
 *  LICENCE:
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.
 *
 */

#include <tulip/SuperGraph.h>
#include <math.h>
#include <string>

//The name of the property that stores the metagraphs of the heirarchy
#define FEATURE_NODE_NAME ("viewMetaGraph")

//metric attributes for leaf ids
#define LEAF_ID_NAME ("leaf id attibute")

//The name of the string proxy that stores the original edge id that
//gave rise to this edge
#define ORIGINAL_EDGE_NAME ("Original Edge ID")

//The name of the graph attibute that contains the id of the node in
//the parent hierarchy.  
#define PARENT_NODE_ATTRIBUTE ("Parent Node")
#define NO_PARENT (0xffffffff)

//The name of the meta graph proxy that tells each node what graph in
//which they are contained
#define CONTAINED_IN_GRAPH ("Node In Graph")

//a define which indicates which is a local property for clones of nodes
#define BRIDGE_PROXY ("Bridge")

//The name of the subgraph that stores the hierarchy
#define HIERARCHY_SUBGRAPH_NAME ("1__Hierarchy")

//The name of the subgraph that stores the final layout
#define FINAL_LAYOUT_SUBGRAPH_NAME ("1__Final Layout")

//The name of a graph that has been untouched
#define UNTOUCHED_SUBGRAPH_NAME ("1__Graph Untouched")

//dataset value for TopoLayout layout function table
//format: topology name; layout high; layout low
#define DATASET_DRAW_OPERATOR ("draw operator")
#define DATASET_CLUSTER_BUSTER ("Hierarchical Cluster Buster")
#define DATASET_FAST_OVERLAPS ("Inl. Fast Ov.")
#define DATASET_TORQUES ("Torques")
#define DATASET_KEEPCOLOURS ("Colour by Topo?")

//Identifiers for the topological features types.  These identifiers are
//used in the cluster/feature names.  Identifiers may not have digits in them
//----------------------------------------------------------------------
#define TREE_IDENT ("Tree ")
#define CONNECTED_IDENT ("Connected ")
#define BICONNECTED_IDENT ("Biconnected ")
#define STRENGTH_IDENT ("Strength ")
#define EMBEDDER_IDENT ("Embedder ")
#define LGL_IDENT ("LGL ")
#define SINGLETONS_IDENT ("Singletons ")
#define UNKNOWN_IDENT ("Unknown ")
//----------------------------------------------------------------------

//a function to determine if there is a high edge to node ratio.
inline 
bool highNodeEdgeRatio (unsigned int nodes, unsigned int edges,
			double nodeEdgeThresh) {
  return (static_cast <double> (edges) >= 
	  nodeEdgeThresh*(nodes*(nodes - 1)/2.0));
}//end highNodeEdgeRatio

//A function to strip out the topo type from a graph name
inline std::string stripTopoType (SuperGraph *graph) {
  std::string name = graph->getAttribute <std::string> ("name");
  unsigned int numPos = 0;
  for (; numPos < name.length(); ++numPos)
    if ((name[numPos] >= '0') && (name[numPos] <= '9')) break; 
  return name.substr (0, numPos);
}//end stripTopoType

//A function to determine if string name is of "type"
inline bool isType (const std::string name, const std::string type) {
  return (name.find (type) == 0) && (name != "") && (type != "");
}//end isType

//A function to determine if "graph" is of "type"
inline bool isType (SuperGraph *graph, const std::string type) {
  std::string name = graph->getAttribute <std::string> ("name");
  return isType (name, type);
}//end isType

#endif
    
