/***************************************************************************
 *   Copyright (C) 2004 by Auber David                                     *
 *   auber@tulip-software.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <set>
#include <deque>
#include <iostream>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include "urlelement.h"

using namespace std;

static const unsigned long timing = 5000;

UrlElement::UrlElement():data(""),serverport(80),fd(-1){
}

UrlElement::UrlElement(const UrlElement &c):
  url(c.url),server(c.server),
  data(""),serverport(c.serverport),fd(-1){
}

void UrlElement::closeFD() {
  if (fd!=-1) close(fd);
  fd=-1;
}

void UrlElement::fill(string &result) {
  char buf[2049];
  memset((char *)buf,0,2049);
  int downloadTry=0;
  fd_set readfs;
  struct timeval tv;
  
  // if we haven't gotten any data for 5*select-time then we aren't getting 
  // any more.
  //load the entire header
  while (downloadTry++ < 5) {
    // check for more data
    FD_ZERO(&readfs);
    FD_SET(fd, &readfs);
    tv.tv_usec = 0;
    tv.tv_sec = 2;
    if (select(FD_SETSIZE, &readfs, NULL, NULL, &tv) == -1) break;
    // if no data yet - then try again; but prepare the timeval again
    if (!FD_ISSET(fd, &readfs)) continue;
    // recv returns <0 on error 
    if (recv(fd, buf, 2047, 0) < 0) break;
    // ignore blank recieves
    if (!strlen(buf)) continue;
    // we got data, so..
    downloadTry = 0;
    result += buf;
    memset((char *)buf,0,2049);
  }
}

void UrlElement::clear() {
  data = "";
}

void UrlElement::load() {
  closeFD();
  fd = siteconnect(server,url,serverport,false);
  if (fd == -1) return;
  fill(data);
  closeFD();
}

bool UrlElement::isHtmlPage() {
  closeFD();
  fd = siteconnect(server,url,serverport,true);
  if (fd == -1) return false;
  string tmp;
  fill(tmp);
  closeFD();
  unsigned int loc = tmp.find( "text/html", 0 );
  if( loc != string::npos )
    return true;
  else
    return false;
}

int UrlElement::siteconnect(const string &server, const string &url,const int serverport, bool headonly) {
  cerr << __PRETTY_FUNCTION__ << endl;
  int serversock, mysock;
  struct hostent *h;
  struct sockaddr_in myaddr, serveraddr;
  char request[8192], *method = NULL;
  
  // check that we actually got data..
  if (server.empty()) return -1;
	
  h = gethostbyname(server.c_str());
  if (!h) {
    cerr << "siteconnect - could not resolve host: "<< server << endl;
    return -1;
  }

  // respect the timing policy!
  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = timing;
  nanosleep(&ts,NULL);
	
  serveraddr.sin_family = h->h_addrtype;
  memcpy((char *) &serveraddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
  serveraddr.sin_port = htons(serverport);
	
  mysock = socket(AF_INET, SOCK_STREAM, 0);
  myaddr.sin_family = AF_INET;
  myaddr.sin_port = htons(0);
  myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  bind(mysock, (struct sockaddr *) &myaddr, sizeof(myaddr));
	
  serversock = connect(mysock, (struct sockaddr *) &serveraddr, sizeof(serveraddr));
  if (serversock < 0) {
    cerr << "siteconnect - could not connect to server " << server;
    cerr << " at port " << serverport;
    cerr << ": " << strerror(errno) << endl;
    return -1;
  }
	
  // now we have to send the actual request!
  if (headonly) {
    method = "HEAD";
  } else {
    method = "GET";
  }

  // prefix the url with / if it doesen't start with it..
  if (url.empty() || url.c_str()[0] != '/') {
    snprintf(request,8190,
	     "%s /%s HTTP/1.0\nHost: %s\nUser-Agent: TulipBot\nReferer: http://%s/%s\n\n",
	     method,url.c_str(),server.c_str(),server.c_str(),url.c_str());
  } else {
    snprintf(request,8190,
	     "%s %s HTTP/1.0\nHost: %s\nUser-Agent: TulipBot\nReferer: http://%s%s\n\n",
	     method,url.c_str(),server.c_str(), server.c_str(),url.c_str());
  }

  write(mysock,request,strlen(request));
	
  return mysock;
} /* end, siteconnect */
