/***************************************************************************
 *   Copyright (C) 2004 by Auber David                                     *
 *   auber@tulip-software.org                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <iostream>
#include <string>
#include <ctype.h>
#include <deque>
#include <set>
#include <netdb.h>
#include <tulip/TlpTools.h>
#include "webcrawler.h"
#include "urlelement.h"

using namespace std;
//========================================================
void WebCrawler::addNode(const UrlElement &url) {
  if (nodes.find(url)==nodes.end()) {
    node n=graph->addNode();
    labels->setNodeValue(n,url.server + url.url);
    nodes[url]=n;
  }
}
//========================================================
void WebCrawler::addEdge(const UrlElement &source,const UrlElement &target) {
  addNode(source);
  addNode(target);
  graph->addEdge(nodes[source],nodes[target]);
}
//========================================================
WebCrawler::WebCrawler(const vector<UrlElement> &mySite, SuperGraph *graph,unsigned int maxSize) {
  this->graph = graph;
  this->maxSize = maxSize;
  labels = graph->getProperty<StringProxy>("URL");
  vector<UrlElement>::const_iterator it=mySite.begin();
  while(it!=mySite.end()) {
    toVisit.push_back(*it);
    addNode(*it);
    ++it;
  }
}
//========================================================
void WebCrawler::start() {
  UrlElement url;
  while (nextUrl(url) && graph->numberOfNodes()<maxSize) {
    if (url.isHtmlPage()) {
      cout << "Visiting : " << url.server << url.url << " ..." << flush;
      url.load();
      parseHtml(url);
      url.clear();
      cout << " done" << endl << flush;
    } else
      cout << "Omitting : " << url.server << url.url << " ==> [not html]"<< endl;
  }
}
//========================================================
bool WebCrawler::nextUrl(UrlElement & url) {
  //cerr << __PRETTY_FUNCTION__ << endl;
    while(!toVisit.empty()) {
      url = toVisit.front();
      toVisit.pop_front();
      if (visited.find(url)==visited.end()) {
	visited.insert(url);
	return true;
      }
    }
    return false;
}
//========================================================
void WebCrawler::findAndTreatUrls(const string&lowercase, const string&balise, const UrlElement &url) {
  unsigned int len = lowercase.length();
  while ( len != string::npos ) {
    len = lowercase.rfind(balise, len );
    if( len != string::npos ) {
      len --;
      unsigned int start = lowercase.find_first_of("=", len );
      start = lowercase.find_first_of("\"", start );
      ++start;
      unsigned int end = lowercase.find_first_of("\"", start );
      string newurl = url.data.substr(start,end-start);
      parseUrl(newurl, url);
    }
  }
}
//========================================================
// parsehtml - parse the given html and add to queue
void WebCrawler::parseHtml (const UrlElement &url) {
  //cerr << __PRETTY_FUNCTION__ << endl << flush;
    if (url.data.empty()) return;
    string lowercase(url.data);
    for (unsigned int i = 0; i< lowercase.length(); ++i)
      lowercase[i] = tolower(lowercase[i]);
    findAndTreatUrls(lowercase,"href",url);
    findAndTreatUrls(lowercase,"src",url);
}
//========================================================
void WebCrawler::addUrl (const UrlElement &url) {
    if (visited.find(url)!=visited.end())
        return;
    toVisit.push_back(url);
}
//========================================================
void WebCrawler::addServer(const string &servername) {
    UrlElement tmpUrl;
    tmpUrl.server = servername;
    tmpUrl.url = "/";
    tmpUrl.serverport = 80;
    tmpUrl.data = "";
    if (visited.find(tmpUrl)!=visited.end())
        return;
    toVisit.push_back(tmpUrl);
}
//========================================================
void WebCrawler::parseUrl (const string &href, const UrlElement &starturl) {
  string lowercase(href);
  for (unsigned int i = 0; i< lowercase.length(); ++i)
    lowercase[i] = tolower(lowercase[i]);
  unsigned int len = lowercase.length();
  if ((lowercase.rfind(".shtm", len) == string::npos) && 
      (lowercase.rfind(".htm", len) == string::npos)) return;
  if (lowercase.rfind("https:", len) != string::npos) return;
  if (lowercase.rfind("javascript:", len) != string::npos) return;
  if (lowercase.rfind("ftp:", len) != string::npos) return;
  if (lowercase.rfind("sftp:", len) != string::npos) return;
  if (lowercase.rfind("gopher:", len) != string::npos) return;
  if (lowercase.rfind("mms:", len) != string::npos) return;
  if (lowercase.rfind("mailto:", len) != string::npos) {
    cerr << "email : " << lowercase << endl;
    return;
  }
  UrlElement newUrl(starturl);
  unsigned int pos=0;
  bool host = false;
  pos = lowercase.rfind("http://", len );
  if (pos == string::npos)
    pos=0;
  else {
    host=true;
    pos+=7;
  }
  if (host) {
    unsigned int endhost = lowercase.find_first_of("/ ",pos);
    if (endhost == string::npos)
      endhost=len;
    string hostname = href.substr(pos,endhost-pos);
    pos = endhost;
    struct hostent *h = NULL;
    h = gethostbyname(hostname.c_str());
    if (!h)
      return;
    if (newUrl.server != hostname) return;
    //newUrl.server = hostname;
    //addServer(hostname);
  }
  unsigned int querystart = lowercase.find_first_of("?#",pos);
  if (querystart != string::npos)
    len=querystart;
  string url = href.substr(pos,len-pos);
  if (url.empty())
    url+="/";
  //Manage relative urls
  if (url[0]!='/') {
    string urlinverse(url.rbegin(),url.rend());
    string urlreference(starturl.url);
    unsigned int findUp = urlreference.rfind("/",urlreference.length());
    if (findUp==string::npos) {
      cerr << "bad url reference can't find one /" << endl;
      return;
    }
    urlreference = urlreference.substr(0,findUp+1);
    while(urlinverse.rfind("/..",urlinverse.length())==(urlinverse.length()-3)) {
      urlinverse=urlinverse.substr(0,urlinverse.length()-3);
      url = url.substr(3,url.length());
      unsigned int findUp = urlreference.rfind("/",urlreference.length());
      if (findUp==string::npos) {
	cerr << "bad url reference, to much ../" << endl;
	return;
      }
      urlreference = urlreference.substr(0,findUp);
    }
    url = urlreference + url;
  }
  newUrl.url=url;
  addUrl(newUrl);
  addEdge(starturl, newUrl);
}

