#include <qpopupmenu.h>
#include <qtimer.h>
#include <qtextview.h>
#include <qimage.h>
#include <qmutex.h>

#include "tulip/GlGraphWidget.h"

#include <tulip/SuperGraph.h>
#include <tulip/Iterator.h>
#include <tulip/SelectionProxy.h>
#include <tulip/LayoutProxy.h>
#include <tulip/SizesProxy.h>
#include <tulip/GlGraph.h>

#include "tulip/MouseInterface.h"

#include <tulip/GlHudRect4.h>
#include <tulip/GlHudRect.h>


using namespace std;
using namespace tlp;
QMutex mutex;

QGLFormat GlInit() {
  QGLFormat tmpFormat;
  tmpFormat.setDirectRendering(true);
  tmpFormat.setDoubleBuffer(true);
  tmpFormat.setAccum(false);
  tmpFormat.setStencil(true);
  tmpFormat.setOverlay(false);
  tmpFormat.setDepth(true);
  tmpFormat.setRgba(true);
  tmpFormat.setAlpha(true);
  return tmpFormat;
}

//==================================================
GlGraphWidget::GlGraphWidget(QWidget *parent, const char *name):
  QGLWidget(GlInit(),parent, name),
  drawTimer(new QTimer(this)),
  mouse(0) {

  //  cerr << __PRETTY_FUNCTION__ << endl;
  setAutoBufferSwap(false);  

  TRACE_EXEC();
  makeCurrent();
  connect(drawTimer, SIGNAL(timeout()), this, SLOT(drawGraph()));
  setFocusPolicy(QWidget::StrongFocus);

  composite = new GlADComposite();
}
//==================================================
GlGraphWidget::GlGraphWidget(const GlGraphWidget &w, QWidget *parent, const char *name):
  QGLWidget(GlInit(),parent, name),
  GlGraph(w),
  drawTimer(new QTimer(this)),
  mouse(0) {

  TRACE_EXEC();
  makeCurrent();
  connect(drawTimer, SIGNAL(timeout()), this, SLOT(drawGraph()));
  setFocusPolicy(QWidget::StrongFocus);

  composite = new GlADComposite();
}
//==================================================
GlGraphWidget::~GlGraphWidget() {
  delete drawTimer;
  if (composite != 0 ) {
    composite->reset(false);
    delete composite;
  }
  if (mouse != 0)
    delete mouse;
}
//==================================================
void GlGraphWidget::closeEvent(QCloseEvent *e) {
  emit closed(this);
}
//==================================================
/*
void GlGraphWidget::setGlGraph(GlGraph *g) {
  TRACE_EXEC();
  exit(1);
  setUpdatesEnabled(false);
  if (glGraph != NULL) delete glGraph;
  glGraph = g;
  setUpdatesEnabled(true);
  redraw();
}
*/
//==================================================
// GlGraph *GlGraphWidget::getGlGraph() const {
//   return glGraph;
// }
//==================================================

void GlGraphWidget::makeCurrent() {
  //  cerr << __PRETTY_FUNCTION__ << " (" << (int)this << ")" << endl;
  assert(context()->isValid());
  QGLWidget::makeCurrent();
}
//==================================================
void GlGraphWidget::updateGL() {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  QGLWidget::updateGL();
}
//==================================================
void GlGraphWidget::initializeGL() {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  GlGraph::initializeGL();
}
//==================================================
void GlGraphWidget::redraw() {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  // if (glGraph ==0) return;
  draw();
}
//==================================================
bool GlGraphWidget::timerIsActive() {
  return drawTimer->isActive();
}
//==================================================
int GlGraphWidget::timerStart(int msec, bool sshot) {
  return drawTimer->start(msec, sshot);
}
//==================================================
void GlGraphWidget::timerStop() {
  drawTimer->stop();
}
//==================================================
void GlGraphWidget::mPaint() {
  makeCurrent();
  if (composite!=0) composite->draw(this);
  if (mouse != NULL) mouse->mPaint(this);
}
//==================================================
void GlGraphWidget::clickAt(int x, int y) {
  node tmpNode;
  edge tmpEdge;
  ElementType type;  
  if (doSelect(x, y, type, tmpNode, tmpEdge)) {
    switch(type) {
    case NODE: emit nodeClicked(getSuperGraph(), tmpNode); break;
    case EDGE: emit edgeClicked(getSuperGraph(), tmpEdge); break;
    }
  }
}
//==================================================
void GlGraphWidget::clickAt(const QPoint &pos) { 
  clickAt(pos.x(), pos.y()); 
}
//==================================================
void GlGraphWidget::setSuperGraph(SuperGraph *sg) {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  // if (glGraph == 0) return;
  TRACE_EXEC();
  GlGraph::setSuperGraph(sg);
  centerScene();
  toUpdate=true;
}
//==================================================
// SuperGraph *GlGraphWidget::getSuperGraph() const {
//   if (glGraph != 0) 
//     return getSuperGraph();
//   else 
//     return NULL;
// }
//==================================================
MouseInterface *GlGraphWidget::getMouse() const {
  return mouse;
}
//==================================================
void GlGraphWidget::setMouse(MouseInterface *m) {
  if (mouse != 0) delete mouse;
  mouse = m->clone();
}
//==================================================
//QGLWidget
//==================================================
QImage GlGraphWidget::grabFrameBuffer(bool withAlpha) {
  bool incState = isIncrementalRendering();
  setIncrementalRendering(false);
  this->updateGL();
  QImage img = QGLWidget::grabFrameBuffer(withAlpha);
  setIncrementalRendering(incState);
  return img;
}
//==================================================
//QGLWidget slots
//==================================================
//void GlGraphWidget::initializeGL() {
  //  cerr << __PRETTY_FUNCTION__ << endl;
//  GlGraph::initializeGL();
//}
//==================================================
void GlGraphWidget::paintGL() {
  TRACE_EXEC();
  makeCurrent();
  draw();
  if (!isIncrementalRendering())
     swapBuffers();
  else
    drawGraph();
}
//==================================================
void GlGraphWidget::resizeGL(int w, int h) {
  TRACE_EXEC();
  //if (glGraph != NULL) 
  changeViewport(0,0,w,h);
}
//==================================================
void GlGraphWidget::drawGraph() {
  TRACE_EXEC();
  makeCurrent();
  // if (glGraph != NULL) {
  if (drawPart()) timerStop();
  swapBuffers();
    //}
}
//==========================================================
// FIXME : This piece of code should be removed  
//==========================================================
/*
  void GlGraphWidget::deleteElement(int x, int y) {
  cerr << "[WARNING]: " << __FUNCTION__ << " : deprecated " << endl;
  if (glGraph == NULL) return;
  drawTimer->stop();
  bool result;
  ElementType type;
  node tmpNode;
  edge tmpEdge;
  Observable::holdObservers();
  result = doSelect(x, y, type, tmpNode, tmpEdge);
  bool strahler=isViewStrahler();
  setViewStrahler(false);
  if(result==true) {
    switch(type) {
    case NODE: getSuperGraph()->delNode(tmpNode); break;
    case EDGE: getSuperGraph()->delEdge(tmpEdge); break;
    }
  }
  setViewStrahler(strahler);
  redraw();
  Observable::unholdObservers();
}
*/
//==========================================================
// FIXME : This piece of code should be removed  
//==========================================================
/*
void GlGraphWidget::contextSelect() {
  cerr << "[WARNING]: " << __FUNCTION__ << " : deprecated " << endl;
  if (glGraph == NULL) return;
  Observable::holdObservers();
  bool result;
  ElementType type;
  node tmpNode;
  edge tmpEdge;
  SelectionProxy *elementSelected=getSuperGraph()->getProperty<SelectionProxy>("viewSelection");
  elementSelected->setAllNodeValue(false);
  elementSelected->setAllEdgeValue(false);
  result=doSelect(xContext,yContext,type,tmpNode,tmpEdge);
  if (result==true) {
    switch(type) {
    case NODE: elementSelected->setNodeValue(tmpNode, true); break;
    case EDGE: elementSelected->setEdgeValue(tmpEdge, true); break;
    }
  }
  Observable::unholdObservers();
}
*/
//==========================================================
// FIXME : This piece of code should be removed  
//==========================================================
 /*
void GlGraphWidget::contextProperties() {
  cerr << "[WARNING]: " << __FUNCTION__ << " : deprecated " << endl;
  node tmpNode;
  edge tmpEdge;
  ElementType type;
  
  if (doSelect(xContext, yContext, type, tmpNode, tmpEdge)) {
    switch(type) {
    case NODE: emit nodeClicked(getSuperGraph(), tmpNode); break;
    case EDGE: emit edgeClicked(getSuperGraph(), tmpEdge); break;
    }
  }
}
 */
//==========================================================
// FIXME : This piece of code should be removed  
//==========================================================
 /*
void GlGraphWidget::contextAddRemoveSelection() {
  cerr << "[WARNING]: " << __FUNCTION__ << " : deprecated " << endl;
  if (glGraph == NULL) return;
  Observable::holdObservers();
  bool result;
  ElementType type;
  node tmpNode;
  edge tmpEdge;
  SelectionProxy *elementSelected=getSuperGraph()->getProperty<SelectionProxy>("viewSelection");
  result=doSelect(xContext, yContext, type, tmpNode, tmpEdge);
  if (result==true) {
    switch(type) {
    case NODE: elementSelected->setNodeValue(tmpNode, !elementSelected->getNodeValue(tmpNode)); break;
    case EDGE: elementSelected->setEdgeValue(tmpEdge, !elementSelected->getEdgeValue(tmpEdge)); break;
    }
    redraw();
  }
  Observable::unholdObservers();
}
 */
//=====================================================
// Events 
//=====================================================
void GlGraphWidget::mousePressEvent(QMouseEvent *e) {
  if (mouse != NULL) 
    mouse->mPressEvent(this, e);
  else {
    if (e->button()==QEvent::LeftButton) {
      node tmpNode;
      edge tmpEdge;
      ElementType type;
      if (doSelect(e->x(), e->y(), type, tmpNode,tmpEdge)) {
	switch(type) {
	case NODE: emit nodeClicked(getSuperGraph(), tmpNode); break;
	case EDGE: emit edgeClicked(getSuperGraph(), tmpEdge); break;
	}
      }
    }
  }
}
//================================================
void GlGraphWidget::mouseMoveEvent(QMouseEvent *e) {
  if (mouse != NULL) mouse->mMoveEvent(this, e);
}

void GlGraphWidget::mouseReleaseEvent(QMouseEvent *e) {
  if (mouse != NULL) mouse->mReleaseEvent(this ,  e);
}

void GlGraphWidget::keyPressEvent(QKeyEvent *e) {
  if (mouse != NULL) mouse->keyPressEvent(this, e);
}

void GlGraphWidget::keyReleaseEvent(QKeyEvent *e) {
  if (mouse != NULL) mouse->keyReleaseEvent(this, e);
}

void GlGraphWidget::wheelEvent(QWheelEvent *e) {
  if (mouse != NULL) mouse->wheelEvent(this, e);
}

void GlGraphWidget::addGlAugmentedDisplay(GlAugmentedDisplay *augmentedDisplay, const char* key) {
  composite->addGlAugmentedDisplay(augmentedDisplay, key);
}

void GlGraphWidget::removeGlAugmentedDisplay(GlAugmentedDisplay *augmentedDisplay) {
  composite->deleteGlAugmentedDisplay(augmentedDisplay);
}

void GlGraphWidget::removeGlAugmentedDisplay(const char* key) {
  composite->deleteGlAugmentedDisplay(key);
}

GlAugmentedDisplay* GlGraphWidget::findGlAugmentedDisplay(const char* key) {
  return composite->findGlAugmentedDisplay(key);
}
