//-*-c++-*-
/*
 Author: Sophie BARDET & Alexandre BURNETT
 Email : sophie.bardet@gmail.com, aburnett397@gmail.com
 Last modification : 19/06/2005
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by  
 the Free Software Foundation; either version 2 of the License, or     
 (at your option) any later version.
*/

#ifndef Tulip_SuperGraphMap_H
#define Tulip_SuperGraphMap_H

#include <vector>
#include <tulip/SuperGraphDeco.h>

class Face;

/** \brief interface for a topological graph */
class SuperGraphMap : public SuperGraphDeco {  // non-orient, planaire
  friend class FaceIterator;
  friend class FaceAdjIterator;
  
 public:
  SuperGraphMap(SuperGraph* s);
  virtual ~SuperGraphMap();

  void clear();
  void reverse(const edge e);
  
  //============================================================
  virtual node addNode();
  virtual void addNode(const node n);
  virtual edge addEdge(const node n, const node n2);
  
  virtual void addEdge(const edge e);
  virtual void delNode(const node n); 
  virtual void delAllNode(const node n);
  virtual void delEdge(const edge e);
  virtual void delAllEdge(const edge e);
  virtual void setEdgeOrder(const node n,const std::vector<edge> & s);
  virtual void swapEdgeOrder(const node n ,const edge e1, const edge e2); 

  //============================================================
  /** add an edge, possibility to specify the predecessor in the cycle */
  edge addEdgeMap(const node v, const node w, const edge e1 = edge(), const edge e2 = edge());
  /** return the number of faces */
  unsigned int nbFaces();
  /** return the face which has the higher number of edges */
  Face* getMaxFace() {return max_face;}
  
  /** return an iterator on all faces */
  Iterator<Face*>* getFaces();
  /** return an iterator on the adjacent faces of a node */
  Iterator<Face*>* getFacesAdj(const node);  
  /** return an iterator on all nodes of a face */
  Iterator<node>* getFaceNodes(const Face*) const;
  /** return an iterator on all edges of a face */
  Iterator<edge>* getFaceEdges(const Face*) const;
 
  /** display all faces by list of edges and nodes */
  void displayFaces();
  /** transform the map to be planar */
  bool makePlanar();
  /** if there's a change, update the faces of the map */
  void update();

  /** warning, use it just in the particular cases : biconnected, simple 
   *  split a face in two parts and create a new edge, all the faces aren't updated */
  Face* splitFace(Face* f, const node v, const node w, node n = node());

  /** warning, use it just in the particular cases : biconnected, simple 
   *  merge two faces and detroy edges and one of the two faces, all the faces aren't updated */
  void mergeFaces(Face* f, Face* g);

  /** successor of e in the cycle of n */
  edge succCycleEdge(const edge e, const node n) const;  
  /** predecessor of e in the cycle of n */
  edge precCycleEdge(const edge e, const node n) const;  
  /** successor of w in the cycle of v */
  node succCycleNode(const node v, const node w) const;  
  /** predecessor of w in the cycle of v */
  node precCycleNode(const node v, const node w) const;  
  

  /** return true if v is a node of f*/
  bool contain(Face * f, node v);
  /** return the face of ident attribute equal to id*/
  Face *getIdFace(int id);
  /** return the face containing nodes v and w in that order*/
  Face *getFaceContaining(node v,node w);
 
private:
  /** reference simply */
  mutable Face* max_face;
  /** storage of faces */
  mutable std::vector<Face*> faces; 
  //
  //vector<vector<edge>> faces;
  //hash_map<node, vector<face>>;
  //hash_map<edge, pair<face, face> >;
  /** variable defined to know if the graph has changed */
  bool hasChanged;
  /** function that does realy compute the faces */
  void computeFaces();
};

#endif

