//#include <iostream>
//#include <list>
//#include <algorithm>
//#include <vector>


#include <tulip/BiconnectedTest.h>
#include <tulip/MutableContainer.h>
#include <tulip/MapIterator.h>
#include <tulip/FaceIterator.h>
//#include <tulip/Face.h>
#include <tulip/SuperGraphMap.h>
#include <tulip/PlanarityTest.h>


using namespace std;

//============================================================
// SuperGraphMap
//============================================================
SuperGraphMap::SuperGraphMap(SuperGraph* s):SuperGraphDeco(s){
  hasChanged = true;
}

//============================================================
SuperGraphMap::~SuperGraphMap(){
  //  for(unsigned int i = 0; i< faces.size();i++)
  //    delete faces[i];
}

//============================================================
void SuperGraphMap::clear(){
  graph_component->clear(); hasChanged = true;
}

//============================================================
void SuperGraphMap::reverse(const edge e){
  graph_component->reverse(e); hasChanged = true;
}

//============================================================
node SuperGraphMap::addNode(){
  hasChanged = true; return graph_component->addNode();
}

//============================================================
void SuperGraphMap::addNode(const node n){
  hasChanged = true; graph_component->addNode(n);
}

//============================================================
edge SuperGraphMap::addEdge(const node n, const node n2){
  hasChanged = true; return graph_component->addEdge(n, n2);
}

//============================================================
void SuperGraphMap::addEdge(const edge e){
  hasChanged = true; graph_component->addEdge(e);
}

//============================================================
void SuperGraphMap::delNode(const node n){
  hasChanged = true; graph_component->delNode(n);
}
 
//============================================================
void SuperGraphMap::delAllNode(const node n){
  hasChanged = true; graph_component->delAllNode(n);
}

//============================================================
void SuperGraphMap::delEdge(const edge e){
  hasChanged = true; graph_component->delEdge(e);
}

//============================================================
void SuperGraphMap::delAllEdge(const edge e){
  hasChanged = true; graph_component->delAllEdge(e);
}

//============================================================
void SuperGraphMap::setEdgeOrder(const node n,const std::vector<edge> & s){
  hasChanged = true; graph_component->setEdgeOrder(n,s);
}

//============================================================
void SuperGraphMap::swapEdgeOrder(const node n ,const edge e1, const edge e2){
  hasChanged = true; graph_component->swapEdgeOrder(n, e1, e2);
}

//============================================================
edge SuperGraphMap::addEdgeMap(const node v, const node w, const edge e1, const edge e2){
  assert(isElement(v) && isElement(w));
  
  hasChanged = true;

  vector<edge> vect; // dfinition of the new order of a cycle
  edge e_tmp; // temporary edge 
  
  edge e = addEdge(v, w); // add the edge

  if(!e1.isValid() && !e2.isValid()) return e; // if there's no specified order
  
  
  if(e1.isValid()){ // special order around v
    assert( isElement(e1) && ((source(e1) == v) || (target(e1)== v)) ); 

    Iterator<edge>* it = getInOutEdges(v); // compute a vector without edge e   
    while(it->hasNext()){
      e_tmp = it->next();
      if(e_tmp != e)
	vect.push_back(e_tmp);
    }
    delete it;

    vector<edge>::iterator i = find(vect.begin(), vect.end(), e1);
    if(i+1 == vect.end()) vect.push_back(e);
    else   vect.insert(++i,e);	 
    setEdgeOrder(v,vect);
    vect.clear();
  }
  
  if(e2.isValid()){ //special order around w
    assert(isElement(e2) && ((source(e2) == w) || (target(e2)==w)) );

    Iterator<edge>* it = getInOutEdges(w); // compute a vector without edge e   
    while(it->hasNext()){
      e_tmp = it->next();
      if(e_tmp != e)
	vect.push_back(e_tmp);
    }
    delete it;
    
    vector<edge>::iterator i = find(vect.begin(), vect.end(),e2);
    if(i+1 == vect.end())  vect.push_back(e);
    else  vect.insert(++i,e);	 
    setEdgeOrder(w,vect);
  }
  return e;
}

//============================================================
edge SuperGraphMap::succCycleEdge(const edge e, const node n) const{
  assert(isElement(e) && isElement(n));

  int i = 0;
  edge e1;
  Iterator<edge>* it = getInOutEdges(n);
  while(it->hasNext()){
    e1 = it->next();
    i++;
    if((e == e1) && (it->hasNext())){
      e1 = it->next();
      delete it;
      return e1;
    }
    else if((e == e1) && (i==1)){ // cycle = an edge
      delete it;
      return e1;
    }
  }

  if(e != e1){
    cerr<<"SuperGraphMap succCycleEdge : edge e not in the cycle"<<endl;
    exit(-1);
  }
  delete it;
  
  it = getInOutEdges(n); // the last edge is e, so first edge is returned
  assert(it->hasNext());
  e1 = it->next();
  delete it;
  return e1;
}

//============================================================
edge SuperGraphMap::precCycleEdge(const edge e, const node n) const{
  assert(isElement(e) && isElement(n));
  Iterator<edge>* it = getInOutEdges(n);

  edge prec, e1;
  int i=0;
  bool stop=false;
  
  while(it->hasNext() && (!stop)){
    e1 = it->next();
    i++;
    if(e == e1)
      stop = true;
    else  prec = e1;
  }

  if(e != e1){
    cerr<<"SuperGraphMap precCycleEdge : edge e not in the cycle"<<endl;
    exit(-1);
  }

  if(i!=1){ // return the precedent edge
    delete it;
    return prec;
  }
  else{
    if(!it->hasNext()){ // cycle = an edge
      return e1;
    }
    else{
      while(it->hasNext()) // return the last element
	e1 = it->next();
      delete it;
      return e1;
    }
  } 
} 

//============================================================
node SuperGraphMap::succCycleNode(const node v , const node w) const{
  // even method that succCycleEdge
  assert(isElement(v) && isElement(w));
  
  int i = 0;
  node n;
  Iterator<node>* it = getInOutNodes(v);
  while(it->hasNext()){
    n = it->next();
    i++;
    if((n == w) && (it->hasNext())){
      n = it->next();
      delete it;
      return n;
    }
    else if((n == w)&&(i==1)){
      delete it;
      return n;
    }
  }

  if(w != n){
    cerr<<"SuperGraphMap succCycleNode : node w("<<w.id<<") not in the cycle de v("<<v.id<<")"<<endl;
    exit(-1);
  }
  delete it;
  
  it = getInOutNodes(v);
  assert(it->hasNext());
  n = it->next();
  delete it;
  return n;
}

//============================================================
node SuperGraphMap::precCycleNode(const node v, const node w) const{
  // even method that precCycleEdge
  assert(isElement(v) && isElement(w));
  Iterator<node>* it = getInOutNodes(v);

  node prec, n1;
  int i=0;
  bool stop=false;
  
  while(it->hasNext() && (!stop)){
    n1 = it->next();
    i++;
    if(w == n1)
      stop = true;
    else  prec = n1;
  }

  if(w != n1){
    cerr<<"SuperGraphMap precCycleNode : node w not in the cycle"<<endl;
    exit(-1);
  }

  if(i!=1){
    delete it;
    return prec;
  }
  else{
    if(!(it->hasNext())){
      return n1;
    }
    while(it->hasNext())
      n1 = it->next();
    delete it;
    return n1;
  } 
} 

//============================================================ 
bool SuperGraphMap::makePlanar(){
  if(!PlanarityTest::isPlanar(this)){
    cerr<<" SuperGraphMap makePlanar : the map can't be planar "<<endl;
    return false;
  }
  PlanarityTest::planarEmbedding(this);
  hasChanged = true;
  update();
  return true;
}

//============================================================
void SuperGraphMap::update(){
  if(hasChanged){
    hasChanged = false;
    computeFaces();
  }
}

//============================================================ 
void SuperGraphMap::computeFaces(){ 
  // to change in order to just compute the modifictions

  for(unsigned int i = 0; i< faces.size(); ++i)
    delete (faces[i]);

  faces.clear();

  if(!(PlanarityTest::isPlanar(this))){
    if(!makePlanar())
      exit(-1);
  }

  int fc = 0;
  MutableContainer<int> considered;
  MutableContainer<bool> sens;
  unsigned int max = 0;
  Face* tmp;
  
  considered.setAll(0);
  sens.setAll(false);

  for(int k=0;k<2; k++){
    Iterator<edge> *it = getEdges();
    while (it->hasNext()) {
      edge e = it->next();
      if (considered.get(e.id)<2){
	Face* lf = new Face();
	faces.push_back(lf);
	edge e1 = e;
	node n_tmp, n;

	if (sens.get(e1.id)) n = target(e1);
	else  n = source(e1);

	n_tmp = n;
	do{
	  considered.set(e1.id, considered.get(e1.id) + 1);
	  EdgeMapIterator it(graph_component, e1, n);
	  e1 = it.next();
	  n = opposite(e1,n);
	  faces[fc]->addEdge(e1);
	  if(source(e1) == n)
	    sens.set(e1.id,true);	    
	}
	while((e1 != e) || (n_tmp != n));
	if(faces[fc]->nbEdges()>max){
	  max = faces[fc]->nbEdges();
	  tmp = faces[fc];
	}
	fc++;
      }
    }
    delete it;
  }
  max_face = tmp;
}

//============================================================
unsigned int SuperGraphMap::nbFaces(){
  update();
  return faces.size();
}
//============================================================
Iterator<Face*>* SuperGraphMap::getFaces(){
  update();
  return new FaceIterator(this);  
}

//============================================================
Iterator<Face*>* SuperGraphMap::getFacesAdj(const node n){
  update();
  return new FaceAdjIterator(this, n);  
}

//============================================================
Iterator<node>* SuperGraphMap::getFaceNodes(const Face* f) const{
  return new NodeFaceIterator(this, f);
}

//============================================================
Iterator<edge>* SuperGraphMap::getFaceEdges(const Face* f) const{
  return new EdgeFaceIterator(f);
}

//============================================================
void SuperGraphMap::displayFaces(){
  update();
  cout<<"Faces :"<<endl<<endl;
  Iterator<Face*>* it = getFaces();
  while(it->hasNext()){
    Face* tmp = it->next();
    cout<<"edges : ";
    Iterator<edge>* ite = getFaceEdges(tmp);
    while(ite->hasNext()){
      edge e = ite->next();
      cout<<e.id<<" ";
    }
    delete ite;
    cout<<", ";

    cout<<"nodes : ";
    Iterator<node>* itn = getFaceNodes(tmp);
    while(itn->hasNext()){
      node n = itn->next();
      cout<<n.id<<" ";
    }
    cout<<endl;
    delete itn;
  }
  delete it;
  cout<<endl;
}

//============================================================
Face* SuperGraphMap::splitFace(Face* f, const node v, const node w, node n){
//------------------------------------------------------------
// prconditions : graph must be simple
//   - biconnected, and updated faces -> hasChanged = false;
//   - n : possibility to specify which face is f after the execution,
//         n must be contain in f 
//------------------------------------------------------------ 
  assert(f);
  assert(BiconnectedTest::isBiconnected(graph_component));

  assert( (!existEdge(v, w).isValid()) && (!existEdge(w, v).isValid()) );
  assert(find(faces.begin(), faces.end(),f) != faces.end());
  assert(hasChanged == false);
  
  if(!n.isValid()){  // if n isn't specify, the define a node by default
    Iterator<edge>* it = getFaceEdges(f);
    while(it->hasNext()){
      edge e = it->next();
      
      if(source(e) == v){
	n = target(e);
	break;
      }
      else if(target(e) == v){
	n = source(e);
	break;
      } 
    }
    delete it;
  }

  node first_node ;
  edge first_edge , first_edge_v, first_edge_w;
  edge epred, e;
  bool vfind = false, wfind = false, nfind = false;  
  // to know if n, v, w  ars visited in the face 
  bool vprec = false, wprec = false, nprec = false;
  bool inter = false;   
  // to know if n is in the interval [v, w] or not 

  Iterator<edge>* it = getFaceEdges(f); // face is also 3 edges, in the biconnected case
  if(it->hasNext()){  // initialisation of values
    e = it->next();
    first_edge = e;
    if( (source(e) == v) || (target(e) == v) )
      vprec = true;
    if( (source(e) == w) || (target(e) == w) )
      wprec = true;
    if( (source(e) == n) || (target(e) == n) )
      nprec = true;
    epred = e;
  }
  while(it->hasNext()){ // course of face
    e = it->next();
    if(!vfind){
      if( (source(e) == v) || (target(e) == v) ){
	if(vprec){
	  first_edge_v = epred;
	  vfind = true; vprec = false;
	  if(!first_node.isValid())
	    first_node = v;
	}
	else vprec = true;
      }
      else if(vprec) vprec = false;
    }
    if(!wfind){ 
      if( (source(e) == w) || (target(e) == w) ){
	if(wprec){
	  first_edge_w = epred;
	  wfind = true; wprec = false;
	  if(!first_node.isValid())
	    first_node = w;
	}
	else wprec = true;
      }
      else if(wprec) wprec = false;
    }
    if(!nfind){
      if( (source(e) == n) || (target(e) == n) ){
	if(nprec){
	  if( !vfind && !wfind ) inter = false;
	  else if( vfind && wfind )   inter = false;
	  else inter = true;
	  nfind = true;
	  nprec = false;
	}
	else nprec = true;
      }
      else if(nprec) nprec = false;
    }
    epred = e;
  }
  delete it;

  edge last_edge = e;
  e = first_edge;
  // th end of the course with the first elements
  if(!vfind){
    if( (source(e) == v) || (target(e) == v) ){
      if(vprec){
	first_edge_v = epred;
	vfind = true; vprec = false;
	if(!first_node.isValid())
	  first_node = v;
      }
      else vprec = true;
    }
  }
  if(!wfind){
    if( (source(e) == w) || (target(e) == w) ){
      if(wprec){
	first_edge_w = epred;
	wfind = true; wprec = false;
	if(!first_node.isValid())
	  first_node = w;
      }
      else wprec = true;
    }
  }
  if(!nfind){
    if( (source(e) == n) || (target(e) == n) ){
      if(nprec){
	if( !vfind && !wfind ) inter = false;
	else if( vfind && wfind )   inter = false;
	else inter = true;
	nfind = true;
	nprec = false;
      }
      else nprec = true;
    }
  }

  assert(nfind && vfind && wfind);
  // creation on the new face
  Face* newf = new Face(f);
  // first_edge_v and second_edge_v, is the edges around v on the face
  edge second_edge_v = succCycleEdge(first_edge_v, v);

  // first_edge_w and second_edge_w, is the edges around w on the face
  edge second_edge_w = succCycleEdge(first_edge_w, w);
  
  // add a new arc
  edge newedge = addEdgeMap(v,w,first_edge_v,first_edge_w);
  hasChanged = false;  // to avoid the compute of faces
 
  if(inter == true){
    if(first_node == v){
      f->delEdges(first_edge,first_edge_v); // give the edges between [v, w]
      if(first_edge_w != last_edge)
	f->delEdges(second_edge_w, last_edge);
      f->addEdge(newedge, first_edge_w);

      newf->delEdges(second_edge_v, first_edge_w);
      newf->addEdge(newedge,first_edge_v);
    }
    else{
      newf->delEdges(second_edge_w, first_edge_v);
      newf->addEdge(newedge, first_edge_w);

      f->delEdges(first_edge,first_edge_w);
      if(first_edge_v != last_edge)
	f->delEdges(second_edge_v, last_edge);
      f->addEdge(newedge, first_edge_v);
    }
  }
  else{
    if(first_node == v){
      f->delEdges(second_edge_v, first_edge_w);
      f->addEdge(newedge,first_edge_v);

      newf->delEdges(first_edge,first_edge_v);
      if(first_edge_w != last_edge)
	newf->delEdges(second_edge_w, last_edge);
      newf->addEdge(newedge, first_edge_w);
    }
    else{
      f->delEdges(second_edge_w, first_edge_v);
      f->addEdge(newedge, first_edge_w);

      newf->delEdges(first_edge,first_edge_w);
      if(first_edge_v != last_edge)
	newf->delEdges(second_edge_v, last_edge);
      newf->addEdge(newedge, first_edge_v);
    }
  }

  faces.push_back(newf);   
  return newf;
}

//============================================================
void SuperGraphMap::mergeFaces(Face* f, Face* g){
  //------------------------------------------------------------
  // prconditions : graph must be simple
  //   - biconnected, and updated faces -> hasChanged = false;
  //------------------------------------------------------------ 
  assert(f && g && (f != g));
  assert(BiconnectedTest::isBiconnected(graph_component));
  assert(find(faces.begin(), faces.end(),f) != faces.end());
  assert(find(faces.begin(), faces.end(),g) != faces.end());
  assert(hasChanged == false);

  bool cut = false;
  edge e_first_commun, e_last_commun;
  unsigned int i = 0;
  // search a continuation of common edges of two faces
  Iterator<edge>* itf = getFaceEdges(f);
  bool not_all_on_both=false;
  while(itf->hasNext()){
    edge ef = itf->next();
    
    if(g->containEdge(ef)){ // find an common edge
      e_first_commun = e_last_commun = ef;
      
      while(itf->hasNext()){
	ef = itf->next();
	if(g->containEdge(ef))
	  e_last_commun = ef;
	else {
	  not_all_on_both=true;
	  break;
	}
      }
      if(!itf->hasNext() && i == 0 && !not_all_on_both){
	cerr<<"SuperGraphMap mergeFaces error : all edges are common ?!?"<<endl;
	exit(-1);
      }
      while(itf->hasNext()){
	ef = itf->next();
	if(g->containEdge(ef)){ // find an arc that is the first of common edges
	  e_first_commun = ef;
	  cut = true;
	  break;
	}
      }
      while(itf->hasNext()){ // vrify if there's no other common edges 
	ef = itf->next();
	if(!g->containEdge(ef)){
	  cerr<<"SuperGraphMap mergeFaces error : disjoined continuation of edges non-autorised"<<endl;
	  exit(-1);
	}
      }
    }
    i++;
  }
  delete itf;

  assert(e_first_commun.isValid() && e_last_commun.isValid());

  node n;
  edge e = f->precEdge(e_first_commun);

  // delete common edges on face f
  if(e_first_commun == e_last_commun)
    f->delEdge(e_first_commun);
  else if(!cut){
    f->delEdges(e_first_commun, e_last_commun);
  }
  else{
    edge e_first = f->getFirstEdge();
    f->delEdges(e_first, e_last_commun);
    edge e_last = f->getLastEdge();
    f->delEdges(e_first_commun, e_last);
  }
  
  if(e == precCycleEdge(e_first_commun, source(e_first_commun)))
    n = source(e_first_commun);
  else n = target(e_first_commun);
  
  // add edges of face g
  edge tmp = succCycleEdge(e_first_commun, n);
  while(tmp != e_last_commun){
    f->addEdge(tmp, e);
    e = tmp;
    tmp = g->succEdge(e);
  }

  // destroy face g, common edges, common nodes
  if(e_first_commun == e_last_commun)
    delEdge(e_first_commun);
  else{
    edge first = e_first_commun, last = g->precEdge(first);
    node middle, other;
    if((source(first) == source(last)) || (target(first) == source(last))){
      middle = source(last);
      other = target(last);
    }
    else if((source(first) == target(last)) || (target(first) == target(last))){
      middle = target(last);
      other = source(last);
    }
    else{
      cerr<<"SuperGraphMap mergeFaces : edges discontinuous"<<endl;
      exit(-1);
    }
    
    while(last != e_last_commun){
      first = last;
      last = g->precEdge(last);
      delNode(middle);
      middle = other;
      other = (middle == source(last))? target(last):source(last);
    }
    delNode(middle);
  }
 
  hasChanged = false;

  vector<Face*>::iterator it = find(faces.begin(), faces.end(), g);
  faces.erase(it, it+1);

  delete g;
}

//============================================================
bool SuperGraphMap::contain(Face* f, node v){
  //------------------------------------------------------------
  // prconditions : graph must be simple
  //   - f and v must be a face and a node of graph
  //------------------------------------------------------------ 
  Iterator<node> * itn = getFaceNodes(f);
  while(itn->hasNext())
    if((itn->next())==v)
      return true;
  return false;
}

//============================================================
Face * SuperGraphMap::getIdFace(int id) {
  //---------------------------------------------------------
  // return the face of ident attribute equal to id
  //---------------------------------------------------------
  Iterator<Face *> * itf = getFaces();
  while(itf->hasNext()) {
    Face * f = itf->next();
    if((*f).ident==static_cast<unsigned int>(id))
      return f;
  }
  cerr << "la face n'est pas une face du graphe.\n";
  exit(-1);
}

//============================================================
Face * SuperGraphMap::getFaceContaining(node v, node w) {
  //---------------------------------------------------------
  // return the face containing nodes v and w in that order
  //---------------------------------------------------------
  Face * res = 0;
  
  Iterator<Face *> * itf = getFaces();
  while (itf->hasNext() && !res) {
    Face * f = itf->next();
    Iterator<node> * itn = getFaceNodes(f);
    bool first = true;
    bool wWasFirst = false;
    bool done = false;
    while (itn->hasNext() && !done) {
      node n = itn->next();
      if (n == v) {
	if ( (!itn->hasNext() && wWasFirst)  || (itn->hasNext() && itn->next()==w)) 
	  res = f;
	done = true;
      }
      else if (n == w) {
	if (first)
	  wWasFirst = true;
	else done = true;
      }
      first = false;
    } delete itn;
  } delete itf;
  return res;
}

