#include "SuperGraphMapTest.h"
#include <tulip/PlanarityTest.h>

#include <cppunit/TestCaller.h>

#include <cppunit/extensions/HelperMacros.h>
CPPUNIT_TEST_SUITE_REGISTRATION( SuperGraphMapTest );

using namespace std;

//============================================================
// SuperGraphMapTest
//============================================================
void SuperGraphMapTest::build(){
  carte->clear(); edges.clear(); nodes.clear();

  for(unsigned int i = 0; i < 4;i++)
    nodes.push_back(carte->addNode());

  edges.push_back(carte->addEdgeMap(nodes[0], nodes[1])); // 0 -> 1
  edges.push_back(carte->addEdgeMap(nodes[1], nodes[2], edges[0])); // 1 -> 2 et 2 -> 1
  edges.push_back(carte->addEdgeMap(nodes[2], nodes[0], edge(), edges[0])); // 2 -> 0 et 0 -> 2
  edges.push_back(carte->addEdgeMap(nodes[0], nodes[3], edges[0], edge())); // 0 -> 3 et 3 -> 0
  
  edges.push_back(carte->addEdgeMap(nodes[3], nodes[1], edges[3], edges[1])); // 3 -> 1 et 1 -> 3
  edges.push_back(carte->addEdgeMap(nodes[2], nodes[3], edges[2], edges[4])); // 2 -> 3 et 3 -> 2
}

//============================================================
void SuperGraphMapTest::build2(){
  carte->clear(); edges.clear(); nodes.clear();
  
  for(int i = 0; i<6; i++)
    nodes.push_back(carte->addNode());
    
  edges.push_back(carte->addEdge(nodes[0], nodes[1])); 
  edges.push_back(carte->addEdge(nodes[1], nodes[2])); 
  edges.push_back(carte->addEdge(nodes[2], nodes[3])); 
  edges.push_back(carte->addEdge(nodes[4], nodes[3])); 
  
  edges.push_back(carte->addEdge(nodes[4], nodes[5])); 
  edges.push_back(carte->addEdge(nodes[0], nodes[5])); 
}

//============================================================
void SuperGraphMapTest::build3(){
  carte->clear(); edges.clear(); nodes.clear();
  
  for(int i = 0; i<6; i++)
    nodes.push_back(carte->addNode());
    
  edges.push_back(carte->addEdge(nodes[0], nodes[1])); 
  edges.push_back(carte->addEdge(nodes[0], nodes[2])); 
  edges.push_back(carte->addEdge(nodes[0], nodes[3])); 
  edges.push_back(carte->addEdge(nodes[0], nodes[4])); 
  edges.push_back(carte->addEdge(nodes[0], nodes[5])); 
}

//============================================================
bool SuperGraphMapTest::containFace(Face* f){

  Iterator<Face*>* it = carte->getFaces();
  while(it->hasNext()){
    Face * tmp = it->next();
    if(tmp == f) return true;
  }
  delete it;
  return false;
}

//============================================================
void SuperGraphMapTest::testAddEdgeMap(){
  build();
  vector<vector<edge> > cycles(4);
  cycles[0].push_back(edges[0]);      cycles[1].push_back(edges[0]);
  cycles[0].push_back(edges[3]);      cycles[1].push_back(edges[1]);
  cycles[0].push_back(edges[2]);      cycles[1].push_back(edges[4]);

  cycles[2].push_back(edges[1]);      cycles[3].push_back(edges[3]);
  cycles[2].push_back(edges[2]);      cycles[3].push_back(edges[4]);
  cycles[2].push_back(edges[5]);      cycles[3].push_back(edges[5]);
  
  for(unsigned int i = 0; i<nodes.size(); i++){
    vector<edge> tmp;
    Iterator<edge>* it = carte->getInOutEdges(nodes[i]);
    while(it->hasNext()){
      edge e = it->next();
      tmp.push_back(e);
    }
    delete it;
    
    CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap cycle ",tmp == cycles[i]);
  }
  
  CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap dispositif decorateur ",carte->numberOfNodes() == graph->numberOfNodes());
}

//============================================================
void SuperGraphMapTest::testNbFaces(){
  build();
  carte->update();
  CPPUNIT_ASSERT_MESSAGE(" test nbFaces ",4 == carte->nbFaces());
}

//============================================================
void SuperGraphMapTest::testUpdate(){
  build();
  carte->update();
  unsigned int tmp = carte->nbFaces();

  node n = carte->addNode();
  carte->addEdgeMap(nodes[0], n, edges[0]);
  carte->addEdgeMap(nodes[1], n);

  carte->update();

  CPPUNIT_ASSERT_MESSAGE(" test update() ", (tmp+1) == carte->nbFaces());   
}

//============================================================
void SuperGraphMapTest::testSplitFace(){
  build2();
  carte->update();
  Face* tmp;

  Iterator<Face*>* it = carte->getFaces();
  tmp = it->next();
  unsigned int nb = tmp->nbEdges();  

  /*   test 1    */
  Face* f = carte->splitFace(tmp, nodes[0], nodes[3]);
  CPPUNIT_ASSERT_MESSAGE(" test splitFace edge ", carte->isElement(carte->existEdge(nodes[0], nodes[3])));
  CPPUNIT_ASSERT_MESSAGE(" test splitFace ",  nb == (f->nbEdges()+tmp->nbEdges()-2));
  for(unsigned int i = 0; i<7; i++)
    CPPUNIT_ASSERT_MESSAGE(" test splitFace edge exist", carte->isElement(edges[i]));

  /*   test 2    */
  nb = tmp->nbEdges();
  f = carte->splitFace(tmp, nodes[5], nodes[3], nodes[0]);
  CPPUNIT_ASSERT_MESSAGE(" test splitFace edge 2", carte->isElement(carte->existEdge(nodes[5], nodes[3])));
  CPPUNIT_ASSERT_MESSAGE(" test splitFace 2",  nb == (f->nbEdges()+tmp->nbEdges()-2));
  for(unsigned int i = 0; i<8; i++)
    CPPUNIT_ASSERT_MESSAGE(" test splitFace edge exist 2", carte->isElement(edges[i]));


  build2();
  carte->update();
  it = carte->getFaces();
  tmp = it->next();
  nb = tmp->nbEdges();  

  /*   test 3    */
  f = carte->splitFace(tmp, nodes[1], nodes[4], nodes[3]);
  CPPUNIT_ASSERT_MESSAGE(" test splitFace edge 3", carte->isElement(carte->existEdge(nodes[1], nodes[4])));
  CPPUNIT_ASSERT_MESSAGE(" test splitFace 3",  nb == (f->nbEdges()+tmp->nbEdges()-2));
  for(unsigned int i = 0; i<7; i++)
    CPPUNIT_ASSERT_MESSAGE(" test splitFace edge exist 3", carte->isElement(edges[i]));
  

   /*   test 4    */
  nb = tmp->nbEdges();
  tmp = carte->splitFace(f, nodes[1], nodes[5], nodes[4]);
  CPPUNIT_ASSERT_MESSAGE(" test splitFace edge 4", carte->isElement(carte->existEdge(nodes[1], nodes[5])));
  CPPUNIT_ASSERT_MESSAGE(" test splitFace 4",  nb == (f->nbEdges()+tmp->nbEdges()-2));
  for(unsigned int i = 0; i<8; i++)
    CPPUNIT_ASSERT_MESSAGE(" test splitFace edge exist 4", carte->isElement(edges[i]));

}

//============================================================
void SuperGraphMapTest::testMergeFaces(){
  Face* f, *f1, *f2;

  /*  test 1  */
  build2();
  edges.push_back(carte->addEdgeMap(nodes[0], nodes[3], edges[0], edges[3]));
  carte->update();
  unsigned nbFaces = carte->nbFaces();
  Iterator<Face*>* it = carte->getFaces();
  f = it->next();
  f1 = it->next();
  f2 = it->next();
  delete it;
  carte->mergeFaces(f1, f);  //carte->displayFaces();
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces containFace f1 (1)", containFace(f1));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces not_containFace f (1)", !containFace(f));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge nbFaces (1)", nbFaces == carte->nbFaces()+1);
  for(unsigned int i = 3; i<7; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge exist (1)", carte->isElement(edges[i]));
  for(unsigned int i = 0; i<3; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge not_exist (1)", !carte->isElement(edges[i]));
      
  /*  test 2  */
  build2();
  edges.push_back(carte->addEdgeMap(nodes[0], nodes[3], edges[0], edges[3]));
  carte->update();
  it = carte->getFaces();
  f = it->next();
  f1 = it->next();
  f2 = it->next();
  delete it;

  carte->mergeFaces(f2, f); //carte->displayFaces();
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces containFace f2 (2)", containFace(f2));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces containFace f (2)", !containFace(f));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge nbFaces (2)", nbFaces == carte->nbFaces()+1);
  for(unsigned int i = 0; i<6; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge exist (2)", carte->isElement(edges[i]));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge not_exist (2)", !carte->isElement(edges[6]));
     
  /*  test 3  */
  build2();
  edges.push_back(carte->addEdgeMap(nodes[0], nodes[3], edges[0], edges[3]));
  carte->update();
  it = carte->getFaces();
  f = it->next();
  f1 = it->next();
  f2 = it->next();
  delete it;

  carte->mergeFaces(f1, f2); //carte->displayFaces();
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces containFace f1 (3)", containFace(f1));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces not_containFace f2 (3)", !containFace(f2));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge nbFaces (3)", nbFaces == carte->nbFaces()+1);
  for(unsigned int i = 0; i<3; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge exist (3)", carte->isElement(edges[i]));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge exist (3)", carte->isElement(edges[6]));
  for(unsigned int i = 3; i<6; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge not_exist (3)", !carte->isElement(edges[i]));
    
  /*  test 4  : suite de plusieurs arcs*/
  build2();
  nodes.push_back(carte->addNode());
  nodes.push_back(carte->addNode());
  edges.push_back(carte->addEdgeMap(nodes[0], nodes[6], edges[5]));
  edges.push_back(carte->addEdgeMap(nodes[3], nodes[7], edges[2]));
  edges.push_back(carte->addEdgeMap(nodes[7], nodes[6]));

  carte->update();
  nbFaces = carte->nbFaces();
  it = carte->getFaces();
  f = it->next();
  f1 = it->next();
  f2 = it->next();
  delete it;

  carte->mergeFaces(f1, f2); 
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces containFace f1 (4)", containFace(f1));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces not_containFace f2 (4)", !containFace(f2));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge nbFaces (4)", nbFaces == carte->nbFaces()+1);
  for(unsigned int i = 0; i<6; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge exist (4)", carte->isElement(edges[i]));  
  for(unsigned int i = 6; i<9; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces edge not_exist (4)", !carte->isElement(edges[i]));
  for(unsigned int i = 0; i<6; i++)
    CPPUNIT_ASSERT_MESSAGE(" test mergeFaces node exist (4)", carte->isElement(nodes[i]));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces node not_exist 1 (4)", !carte->isElement(nodes[6]));
  CPPUNIT_ASSERT_MESSAGE(" test mergeFaces node not_exist 2 (4)", !carte->isElement(nodes[7]));
}

//============================================================
void SuperGraphMapTest::testSuccCycleEdge(){
  build3();

  vector<edge> cycles;
  cycles.push_back(edges[0]);
  cycles.push_back(edges[1]);
  cycles.push_back(edges[2]);
  cycles.push_back(edges[3]);
  cycles.push_back(edges[4]); 
  cycles.push_back(edges[0]);
  
  unsigned int i = 0;
  Iterator<edge>* it = carte->getInOutEdges(nodes[0]);
  edge e = it->next();
  CPPUNIT_ASSERT_MESSAGE(" test SuccCycleEdge deb",e == cycles[0]);
  do{
    e = carte->succCycleEdge(e,nodes[0]);
    i++;
    CPPUNIT_ASSERT_MESSAGE(string(" test SuccCycleEdge cycle "),e == cycles[i]);
  }
  while(e != edges[0] && i<4);
  e = carte->succCycleEdge(e,nodes[0]);
  i++;
  CPPUNIT_ASSERT_MESSAGE(" test SuccCycleEdge end",e == cycles[i]);

  delete it;

}
   
//============================================================
void SuperGraphMapTest::testPrecCycleEdge(){
  build3();

  vector<edge> cycles;
  cycles.push_back(edges[0]);
  cycles.push_back(edges[4]);
  cycles.push_back(edges[3]);
  cycles.push_back(edges[2]);
  cycles.push_back(edges[1]);
  cycles.push_back(edges[0]);
  
  unsigned int i = 0;
  Iterator<edge>* it = carte->getInOutEdges(nodes[0]);
  edge e = it->next();
  CPPUNIT_ASSERT_MESSAGE(" test SuccCycleEdge deb",e == cycles[0]);
  do{
    e = carte->precCycleEdge(e,nodes[0]);
    i++;
    CPPUNIT_ASSERT_MESSAGE(string(" test SuccCycleEdge cycle "),e == cycles[i]);
  }
  while((e != edges[0]) && (i<4));
  e = carte->precCycleEdge(e,nodes[0]);
  i++;
  CPPUNIT_ASSERT_MESSAGE(" test SuccCycleEdge end",e == cycles[i]);

  delete it;
}  

//============================================================
void SuperGraphMapTest::testComputeFaces(){
  build();
  carte->update();

  int fc = carte->nbFaces();
  int e = carte->numberOfEdges();
  int n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 1", fc == e - n + 2);

  build2();
  carte->update();

  fc = carte->nbFaces();
  e = carte->numberOfEdges();
  n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 2", fc == e - n + 2);

  build3();
  carte->update();

  fc = carte->nbFaces();
  e = carte->numberOfEdges();
  n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 3 ", fc == e - n + 2);
}

//============================================================
CppUnit::Test* SuperGraphMapTest::suite(){
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite;
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testAddEdgeMap", &SuperGraphMapTest::testAddEdgeMap));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testNbFaces", &SuperGraphMapTest::testNbFaces));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testUpdate", &SuperGraphMapTest::testUpdate));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testSuccCycleEdge", &SuperGraphMapTest::testSuccCycleEdge));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testPrecCycleEdge", &SuperGraphMapTest::testPrecCycleEdge));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testSplitFace", &SuperGraphMapTest::testSplitFace));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
  		   "testMergeFaces", &SuperGraphMapTest::testMergeFaces));
  suiteOfTests->addTest(new CppUnit::TestCaller<SuperGraphMapTest>(
		   "testComputeFaces", &SuperGraphMapTest::testComputeFaces));
  return suiteOfTests;
}

