
/** \addtogroup layout */
/*@{*/
/// AreaRINGS.cpp -- An adaptation of the AreaRINGS algorithm.
/**
 * Integration in Tulip by D. Archambault
 *
 */
#include <tulip/TulipPlugin.h>
#include <deque>
#include <vector>
#if (__GNUC__ < 3)
#  include <hash_map>
#else
#  include <ext/hash_map>
#endif

class AreaRINGS : public Layout {
 public:
	 inline AreaRINGS (const PropertyContext &context):Layout (context) {}
	 inline ~AreaRINGS() {}
	 bool run();
	 bool check(std::string &);
	 void reset();
 private:
	 MetricProxy *radius;
	 LayoutProxy *rootRelative;
	 LayoutProxy *rootRelativeOffset;
	 __gnu_cxx::hash_map<edge, std::vector<edge> > originalEdges;

	 stdext::hash_map<node, std::deque<node> > chains;
	 void setEdgeBend (node root, node child, unsigned int level);
	 void layoutChain (node &root, node &chainNode, double rootRadius,
			   unsigned int level = 0);
	 void layoutFork (node &root, node &left, node &right,
			  double rootRadius, unsigned int level = 0);
	 void handleChain (node &root, std::deque<node> &chain);
	 double setPlacedChildrenOrder (node root, 
					std::vector<node> &placedChildren);
	 void layoutUsingRatio (node &root, std::deque<node> &children,
				double centreRadius, 
				bool edgeOrderChildren);
	 void AreaRINGSLayout (node curRoot, std::deque<node> *chainNodes);
	 void setGlobalCoords (node curRoot, edge cameFromEdge, Coord curPos);
};
