#include "configs.h"
#include "calcFuncs.h"
#include "graph.hpp"
#include "LGL.h"

using namespace std;

int runLayout( int argc, char ** argv, Graph_t &g, NodeContainer &nodes );

bool LGL::run () {
  //turn tulip into a boost graph
  Graph_t::boost_graph bg;
  Graph_t::vertex_index_map idmap;
  unsigned int index = 0;
  node indexToNode[superGraph->numberOfNodes()];
  Iterator<node> *tulipNodes = superGraph->getNodes();
  while (tulipNodes->hasNext()) {
    node curNode = tulipNodes->next();
    ostringstream convert;
    convert << curNode.id << flush;
    idmap.createMap (convert.str(), index);
    indexToNode[index] = curNode;
    ++index;
  } delete tulipNodes; 
  
  Iterator<edge> *edges = superGraph->getEdges();
  while (edges->hasNext()) {
    edge curEdge = edges->next();
    ostringstream source, target;
    source << superGraph->source (curEdge).id << flush;
    target << superGraph->target (curEdge).id << flush; 
    add_edge (idmap.findLatter (source.str()), 
	      idmap.findLatter (target.str()), bg);
  } delete edges;
  Graph_t g;
  g.boostGraph(bg);
  g.hasWeights (false);
  g.vertexIdMap (idmap);

  // Create the particles now based on the graph.
  NodeContainer nodes( g.vertexCount() );
  // Initialize the ids of the nodes
  for ( NodeContainer::size_type ii=0; ii<nodes.size(); ++ii ) {
    nodes.ids[ii] = g.idFromIndex( ii );
    nodes[ii].id( g.idFromIndex( ii ) );
  }

  //see if the user passed in an lgl root node
  DataSet &attributes = superGraph->getAttributes();
  unsigned int lglrootint;
  string lglroot;
  bool passedLGLRoot = attributes.get ("lglroot", lglrootint);
  if (passedLGLRoot) {
    ostringstream rootstrstr;
    rootstrstr << lglrootint << flush;
    lglroot = rootstrstr.str();
    bool rootFound = idmap.doesMapExist (lglroot);
    if (!rootFound) {
      cerr << "Passed root " << lglrootint << " not found in graph "
	   << "treating as if a root was not passed." << endl;
      passedLGLRoot = false;
    }//end if
    else cerr << "Using passed root " << lglrootint << "." << endl;
  }//end if

  // This determines the space necessary to place all the nodes.
  prec_t outerRadius = -1.0;
  if ( DIMENSION == 2 ) { 
    outerRadius = (prec_t) sqrt( (double) nodes.size() );
  } else if ( DIMENSION == 3 ) {
    outerRadius = (prec_t) pow( (double) nodes.size() , .33333 );
  } else { cerr << "Only 2 or 3 dimensions\n"; exit(EXIT_FAILURE); }
  cerr << "Outer radius is set to " << outerRadius << endl;
  
  ostringstream strOuterRadius;
  strOuterRadius << outerRadius << flush;

  unsigned int numArgs = 3;
  if (passedLGLRoot) numArgs += 2;
  char *charArguments [numArgs];
  char argumentsArray [numArgs*20];
  for (unsigned int i = 0; i < numArgs; i++)
    charArguments[i] = argumentsArray + i*20;
  sprintf (charArguments[0], "lglayout");
  sprintf (charArguments[1], "-R");
  sprintf (charArguments[2], strOuterRadius.str().c_str());
  if (passedLGLRoot) {
    sprintf (charArguments[3], "-z");
    sprintf (charArguments[4], lglroot.c_str());
  }//end if
  runLayout (numArgs, charArguments, g, nodes);

  //set the final positions
  for ( NodeContainer::size_type ii=0; ii<nodes.size(); ++ii ) 
    layoutProxy->setNodeValue (indexToNode[ii], Coord (nodes[ii].X()[0],
	                                               nodes[ii].X()[1]));
  return true;
}//end run

bool LGL::check (std::string &msg) { return true; }

void LGL::reset () {}

LAYOUTPLUGIN(LGL, "LGL", "Daniel Archambault", "20/06/2005", "Ok", "0", "1");

